import numpy as np

from pythran.config import cfg
from pythran.tests import TestEnv
from pythran.typing import NDArray

import unittest

try:
    np.float128
    has_float128 = True
except AttributeError:
    has_float128 = False


class TestComplex(TestEnv):

    """ Check complex support in Pythran. """

    def test_complex_limited_range(self):
        """ Check complex computation is the same as numpy for corner case. """
        # see -fcx-limited-range
        if cfg.getboolean('pythran', 'complex_hook'):
            self.run_test("""
                def test_complex_limited_range(a, b):
                    return a * b""",
                          complex(-4, np.nan), complex(4, -np.inf),
                          test_complex_limited_range=[complex, complex])

    def test_complex128_to_complex64(self):
        self.run_test("""
            import numpy as np
            def complex128_to_complex64(a):
                return np.complex64(a)""",
            complex(-4.4, 4.4),
            complex128_to_complex64=[complex])

    def test_conjugate(self):
        """
        Check complex conjugate.

        Checked for:
            * Method and numpy function call
            * conj and conjugate for each of them
            * complex and array (1 and 2 D)
        """
        self.run_test("""
            def test_conjugate(c, a, a2d):
                import numpy as np
                return (np.conj(c), np.conj(a), a2d.conj(),
                        np.conjugate(c), np.conjugate(a), a2d.conjugate())
                      """,
                      3 + 2j, np.array([3 + 2j]), np.array([[3 + 2j]]),
                      test_conjugate=[np.complex128,
                                      NDArray[np.complex128, :],
                                      NDArray[complex, :, :]])

    def test_complex_array_abs(self):
        self.run_test('def test_complex_array_abs(a): import numpy as np ; return np.abs(a)',
                      np.array([[3 + 2j]]),
                      test_complex_array_abs=[NDArray[complex, :, :]])

    def test_complex_floordiv(self):
        self.run_test('def complex_floordiv(x): import numpy as np; return np.floor_divide(x, 2 + 2j)',
                      3.5 - 3.5j,
                      complex_floordiv=[complex])

    def test_complex_array_sqr(self):
        self.run_test('def test_complex_array_sqr(a): return a ** 2',
                      np.array([[3 + 2j]]),
                      test_complex_array_sqr=[NDArray[complex, :, :]])

    def test_complex_array_mul_i(self):
        self.run_test('def test_complex_array_mul_i(e): return e + 1j * e',
                      np.array([[3.,2.,4.]]),
                      test_complex_array_mul_i=[NDArray[float, :, :]])

    def test_non_complex_array_real_imag(self):
        self.run_test('def test_non_complex_array_real_imag(e): return e.real + e.imag',
                      np.array([[3.,2.,4.]]),
                      test_non_complex_array_real_imag=[NDArray[float, :, :]])

    def test_complex_array_real_imag(self):
        self.run_test('def test_complex_array_real_imag(e): return e.real + e.imag',
                      np.array([[3.,2.,4.]], dtype=complex),
                      test_complex_array_real_imag=[NDArray[complex, :, :]])

    def test_complex_sum_different_types(self):
        self.run_test('def test_complex_different_types(a,b): return a + b',
                      np.array([[3 + 2j]],dtype=np.complex64),np.array([[8 + 1j]],dtype=np.complex128),
                      test_complex_different_types=[NDArray[np.complex64, :, :],NDArray[np.complex128, :, :]])

    def test_complex_sum_same_types(self):
        self.run_test('def test_complex_same_types(a): return a + a',
                      np.array([[3 + 2j]],dtype=np.complex64),
                      test_complex_same_types=[NDArray[np.complex64, :, :]])

    def test_complex_array_real_assign(self):
        self.run_test('def test_complex_array_real_assign(a): a.real = 1; return a',
                      np.array([[3 + 2j, 2, 1, 0]] * 3,dtype=np.complex64),
                      test_complex_array_real_assign=[NDArray[np.complex64, :, :]])

    def test_complex_array_gexpr_real_assign(self):
        self.run_test('def test_complex_array_gexpr_real_assign(a): a.real[1:] = 1; return a',
                      np.array([[3 + 2j, 2, 1, 0]] * 3,dtype=np.complex64),
                      test_complex_array_gexpr_real_assign=[NDArray[np.complex64, :, :]])

    def test_complex_array_iexpr_real_assign(self):
        self.run_test('def test_complex_array_iexpr_real_assign(a): a.real[1] = 1; return a',
                      np.array([[3 + 2j, 2, 1, 0]] * 3,dtype=np.complex64),
                      test_complex_array_iexpr_real_assign=[NDArray[np.complex64, :, :]])

    def test_complex_broadcast_scalar0(self):
        self.run_test('def complex_broadcast_scalar0(x): return x + 1.5, 1.3 +x, 3.1 - x, x - 3.7, x * 5.4, 7.6 * x',
                      5.1 + 3j,
                      complex_broadcast_scalar0=[complex])

    def test_complex_broadcast_scalar1(self):
        self.run_test('def complex_broadcast_scalar1(x): return x + 1.5, 1.3 +x, 3.1 - x, x - 3.7, x * 5.4, 7.6 * x',
                      np.complex64(5.1 + 3j),
                      complex_broadcast_scalar1=[np.complex64])

    def test_complex_array_imag_assign(self):
        self.run_test('def test_complex_array_imag_assign(a): a.imag = 1; return a',
                      np.array([[3 + 2j, 2, 1, 0]] * 3,dtype=np.complex64),
                      test_complex_array_imag_assign=[NDArray[np.complex64, :, :]])

    def test_complex_array_gexpr_imag_assign(self):
        self.run_test('def test_complex_array_gexpr_imag_assign(a): a.imag[1:] = 1; return a',
                      np.array([[3 + 2j, 2, 1, 0]] * 3,dtype=np.complex64),
                      test_complex_array_gexpr_imag_assign=[NDArray[np.complex64, :, :]])

    def test_complex_array_iexpr_imag_assign(self):
        self.run_test('def test_complex_array_iexpr_imag_assign(a): a.imag[1] = 1; return a',
                      np.array([[3 + 2j, 2, 1, 0]] * 3,dtype=np.complex64),
                      test_complex_array_iexpr_imag_assign=[NDArray[np.complex64, :, :]])

    def test_complex_array_expr_imag(self):
        self.run_test('def test_complex_array_expr_imag(a): return (2.j*a).imag',
                      np.array([[3 + 2j, 2, 1, 0]] * 3,dtype=np.complex64),
                      test_complex_array_expr_imag=[NDArray[np.complex64, :, :]])

    def test_complex_array_expr_real(self):
        self.run_test('def test_complex_array_expr_real(a): return (2+a).real',
                      np.array([[3 + 2j, 2, 1, 0]] * 3,dtype=np.complex64),
                      test_complex_array_expr_real=[NDArray[np.complex64, :, :]])

    @unittest.skipIf(not has_float128, "not float128")
    def test_complex256_array0(self):
        self.run_test('def complex256_array0(x): import numpy as np; return np.cos(x * 2j)',
                      np.array([1.2,3.1], dtype=np.complex256) ** 6,
                      complex256_array0=[NDArray[np.complex256, :]])

    @unittest.skipIf(not has_float128, "not float128")
    def test_complex256_array1(self):
        self.run_test('def complex256_array1(x): import numpy as np; return (x * 2j)**2',
                      np.array([1.2,3.1], dtype=np.complex256) ** 6,
                      complex256_array1=[NDArray[np.complex256, :]])

    @unittest.skipIf(not has_float128, "not float128")
    def test_complex256_array2(self):
        self.run_test('def complex256_array2(x): import numpy as np; return np.ones(x, dtype=np.complex256)',
                      10,
                      complex256_array2=[int])

    @unittest.skipIf(not has_float128, "not float128")
    def test_complex256_array3(self):
        self.run_test('def complex256_array3(x): return x.real, x.imag',
                      np.array([2j, 2], dtype=np.complex256)** 5,
                      complex256_array3=[NDArray[np.complex256, :]])

    @unittest.skipIf(not has_float128, "not float128")
    def test_complex256_array4(self):
        self.run_test('def complex256_array4(x): return x.conj(), x.sum()',
                      np.array([2j, 2], dtype=np.complex256)** 7,
                      complex256_array4=[NDArray[np.complex256, :]])

    @unittest.skipIf(not has_float128, "not float128")
    def test_complex256_array5(self):
        self.run_test('def complex256_array5(x): return x',
                      np.complex256(1 + 1j),
                      complex256_array5=[np.complex256])




#pythran export lapl2d(float[,],float[,],int)
#runas import numpy as np; x = np.arange(16.).reshape(4,4); lapl2d(x, x.T, 3)
def lapl2d(In,Out,niter):
    siz=In.shape[0]        
    h2= (1./siz)**2
    for it in range(0,niter):
        Out[1:siz-1,1:siz-1]= h2*(
            In[0:siz-2,1:siz-1 ] + In[1:siz-1,0:siz-2]-
            4.0*In[1:siz-1,1:siz-1]+
            In[2:siz,1:siz-1]+In[1:siz-1,2:siz])
        In,Out=Out,In



def omp_task_if(condition_false=False):
    from time import sleep
    count = 0
    result = 0

    if 'omp parallel':
        if 'omp single':
            if 'omp task if(condition_false) shared(count, result)':
                sleep(0.5)
                result = int(count == 0)

            count = 1

    return result



from subprocess import check_call
import os
import re
import shutil
import sys
import sysconfig
import unittest

cwd = os.path.dirname(__file__)
python_version = "python{}.{}".format(sys.version_info.major,
                                      sys.version_info.minor)

def find_so(name, path):
    for root, dirs, files in os.walk(path):
        for filename in files:
            if re.match(name, filename):
                return os.path.join(root, filename)

class TestDistutils(unittest.TestCase):

    def test_setup_build(self):
        check_call(['python', 'setup.py', 'build'],
                   cwd=os.path.join(cwd, 'test_distutils'))
        check_call(['python', 'setup.py', 'install', '--prefix=demo_install'],
                   cwd=os.path.join(cwd, 'test_distutils'))

        base = os.path.join(cwd, 'test_distutils', 'demo_install',)
        libdir = os.path.join(base, 'lib')
        if not os.path.isdir(libdir):
            libdir = os.path.join(base, 'lib64')
        check_call(['python', '-c', 'import demo'],
                   cwd=os.path.join(libdir, python_version, 'site-packages'))
        check_call(['python', 'setup.py', 'clean'],
                   cwd=os.path.join(cwd, 'test_distutils'))
        shutil.rmtree(os.path.join(cwd, 'test_distutils', 'demo_install'))
        shutil.rmtree(os.path.join(cwd, 'test_distutils', 'build'))

    def test_setup_sdist_install(self):
        check_call(['python', 'setup.py', 'sdist', "--dist-dir=sdist"],
                   cwd=os.path.join(cwd, 'test_distutils'))
        check_call(['tar', 'xzf', 'demo-1.0.tar.gz'],
                   cwd=os.path.join(cwd, 'test_distutils', 'sdist'))
        check_call(['python', 'setup.py', 'install', '--prefix=demo_install'],
                   cwd=os.path.join(cwd, 'test_distutils', 'sdist', 'demo-1.0'))
        shutil.rmtree(os.path.join(cwd, 'test_distutils', 'sdist'))

    def test_setup_bdist_install(self):
        check_call(['python', 'setup.py', 'bdist', "--dist-dir=bdist"],
                   cwd=os.path.join(cwd, 'test_distutils'))
        dist_path = os.path.join(cwd, 'test_distutils', 'bdist')
        tgz = [f for f in os.listdir(dist_path) if f.endswith(".tar.gz")][0]
        check_call(['tar', 'xzf', tgz], cwd=dist_path)

        demo_so = find_so(r"demo.*\.so", dist_path)
        self.assertIsNotNone(demo_so)
        shutil.rmtree(dist_path)

    def test_setup_wheel_install(self):
        check_call(['python', 'setup.py', 'bdist_wheel', "--dist-dir=bdist_wheel"],
                   cwd=os.path.join(cwd, 'test_distutils_setuptools'))
        dist_path = os.path.join(cwd, 'test_distutils_setuptools', 'bdist_wheel')
        wheel_dir = 'wheeeeeeel'
        whl = [f for f in os.listdir(dist_path) if f.endswith(".whl")][0]
        check_call(['unzip', whl, '-d', wheel_dir], cwd=dist_path)

        demo_so = find_so(r"demo.*\.so", os.path.join(dist_path, wheel_dir))
        self.assertIsNotNone(demo_so)
        shutil.rmtree(dist_path)


    def test_setup_build2(self):
        check_call(['python', 'setup.py', 'build'],
                   cwd=os.path.join(cwd, 'test_distutils_packaged'))
        check_call(['python', 'setup.py', 'install', '--prefix=demo_install2'],
                   cwd=os.path.join(cwd, 'test_distutils_packaged'))

        base = os.path.join(cwd, 'test_distutils_packaged', 'demo_install2',)
        libdir = os.path.join(base, 'lib')
        if not os.path.isdir(libdir):
            libdir = os.path.join(base, 'lib64')
        check_call(['python', '-c', 'import demo2.a'],
                   cwd=os.path.join(libdir, python_version, 'site-packages'))
        check_call(['python', 'setup.py', 'clean'],
                   cwd=os.path.join(cwd, 'test_distutils_packaged'))
        shutil.rmtree(os.path.join(cwd, 'test_distutils_packaged', 'demo_install2'))
        shutil.rmtree(os.path.join(cwd, 'test_distutils_packaged', 'build'))

    def test_setup_sdist_install2(self):
        check_call(['python', 'setup.py', 'sdist', "--dist-dir=sdist2"],
                   cwd=os.path.join(cwd, 'test_distutils_packaged'))
        check_call(['tar', 'xzf', 'demo2-1.0.tar.gz'],
                   cwd=os.path.join(cwd, 'test_distutils_packaged', 'sdist2'))
        check_call(['python', 'setup.py', 'install', '--prefix=demo_install2'],
                   cwd=os.path.join(cwd, 'test_distutils_packaged', 'sdist2', 'demo2-1.0'))
        shutil.rmtree(os.path.join(cwd, 'test_distutils_packaged', 'sdist2'))

    def test_setup_bdist_install2(self):
        check_call(['python', 'setup.py', 'bdist', "--dist-dir=bdist"],
                   cwd=os.path.join(cwd, 'test_distutils_packaged'))
        dist_path = os.path.join(cwd, 'test_distutils_packaged', 'bdist')
        tgz = [f for f in os.listdir(dist_path) if f.endswith(".tar.gz")][0]
        check_call(['tar', 'xzf', tgz], cwd=dist_path)

        demo_so = find_so(r"a.*\.so", dist_path)
        self.assertIsNotNone(demo_so)
        shutil.rmtree(dist_path)

    def test_setup_build3(self):
        check_call(['python', 'setup.py', 'build'],
                   cwd=os.path.join(cwd, 'test_distutils_numpy'))
        check_call(['python', 'setup.py', 'install', '--prefix=demo_install3'],
                   cwd=os.path.join(cwd, 'test_distutils_numpy'))

        base = os.path.join(cwd, 'test_distutils_numpy', 'demo_install3',)
        libdir = os.path.join(base, 'lib')
        if not os.path.isdir(libdir):
            libdir = os.path.join(base, 'lib64')
        check_call(['python', '-c', 'import a'],
                   cwd=os.path.join(libdir, python_version, 'site-packages',
                                    'demo3'))
        check_call(['python', 'setup.py', 'clean'],
                   cwd=os.path.join(cwd, 'test_distutils_numpy'))
        shutil.rmtree(os.path.join(cwd, 'test_distutils_numpy', 'demo_install3'))
        shutil.rmtree(os.path.join(cwd, 'test_distutils_numpy', 'build'))

    def test_setup_sdist_install3(self):
        check_call(['python', 'setup.py', 'sdist', "--dist-dir=sdist3"],
                   cwd=os.path.join(cwd, 'test_distutils_numpy'))
        check_call(['tar', 'xzf', 'demo3-1.0.tar.gz'],
                   cwd=os.path.join(cwd, 'test_distutils_numpy', 'sdist3'))
        check_call(['python', 'setup.py', 'install', '--prefix=demo_install3'],
                   cwd=os.path.join(cwd, 'test_distutils_numpy', 'sdist3', 'demo3-1.0'))
        shutil.rmtree(os.path.join(cwd, 'test_distutils_numpy', 'sdist3'))

    def test_setup_bdist_install3(self):
        check_call(['python', 'setup.py', 'bdist', "--dist-dir=bdist"],
                   cwd=os.path.join(cwd, 'test_distutils_numpy'))
        dist_path = os.path.join(cwd, 'test_distutils_numpy', 'bdist')
        tgz = [f for f in os.listdir(dist_path) if f.endswith(".tar.gz")][0]
        check_call(['tar', 'xzf', tgz], cwd=dist_path)

        demo_so = find_so(r"a.*\.so", dist_path)
        self.assertIsNotNone(demo_so)
        shutil.rmtree(dist_path)



#pythran export next_pythagorean_triples(int64[:,:])
#runas import numpy as np; next_pythagorean_triples(np.array([[3, 4, 5]], dtype=np.int64))
import numpy as np
def next_pythagorean_triples(previous):
    matrices = np.array(
        [[-1, 2, 2],
         [-2, 1, 2],
         [-2, 2, 3],
         [1, 2, 2],
         [2, 1, 2],
         [2, 2, 3],
         [1, -2, 2],
         [2, -1, 2],
         [2, -2, 3]])

    next_triples = np.transpose(np.dot(matrices, np.transpose(previous)))
    next_triples = next_triples.reshape((3 * previous.shape[0], previous.shape[1]))
    return next_triples



import numpy as np
#pythran export sqr_dists(float[:,:], float[:,:])
#pythran export sqr_dists_loops(float[:,:], float[:,:])
#bench u = 300; d = 300; import numpy as np; b = np.ones((u,d)); a = np.ones((u,d)); sqr_dists(a, b)

def sqr_dists(X,Y):
  return np.array([[np.sum( (x-y) ** 2) for x in X] for y in Y])

def sqr_dists_loops(X,Y):
  m,n = X.shape[0], Y.shape[0]
  D = np.zeros((m,n))
  for i in range(m):
    for j in range(n):
      D[i,j] = np.sum( (X[i] -Y[j]) ** 2)
  return D




from pythran.tests import TestEnv
import warnings
warnings.filterwarnings("ignore",category=DeprecationWarning)


@TestEnv.module
class TestNumpyRandom(TestEnv):

    ###########################################################################
    #Tests for numpy.random.random
    ###########################################################################

    def test_numpy_random0(self):
        """ Check numpy random without arguments. """
        self.run_test("""
            def numpy_random0(n):
                from numpy.random import random
                s = sum(random() for x in range(n))
                return (abs(s / n - .5) < .05)""",
                      10 ** 5, numpy_random0=[int])

    def test_numpy_random1(self):
        """ Check numpy random with size argument. """
        self.run_test("""
            def numpy_random1(n):
                from numpy.random import random
                s = sum(random(size=n))
                return (abs(s / n - .5) < .05)""",
                      10 ** 5, numpy_random1=[int])

    def test_numpy_random2(self):
        """ Check numpy random with shape argument. """
        self.run_test("""
            def numpy_random2(n):
                from numpy.random import random
                from numpy import sum
                s = sum(random((n, n)))
                return (abs(s / (n * n) - .5) < .05)""",
                      10 ** 3, numpy_random2=[int])

    def test_numpy_random3(self):
        """ Check numpy random with constant size argument. """
        self.run_test("""
            def numpy_random3(n):
                from numpy.random import random
                s = sum(random(10 ** 5))
                return (abs(s / n - .5) < .05)""",
                      10 ** 5, numpy_random3=[int])


    ###########################################################################
    #Tests for numpy.random.random_sample
    ###########################################################################

    def test_numpy_random_sample0(self):
        """ Check numpy random_sample without arguments. """
        self.run_test("""
            def numpy_random_sample0(n):
                from numpy.random import random_sample
                s = sum(random_sample() for x in range(n))
                return (abs(s / n - .5) < .05)""",
                      10 ** 5, numpy_random_sample0=[int])

    def test_numpy_random_sample1(self):
        """ Check numpy random_sample with size argument. """
        self.run_test("""
            def numpy_random_sample1(n):
                from numpy.random import random_sample
                s = sum(random_sample(size=n))
                return (abs(s / n - .5) < .05)""",
                      10 ** 5, numpy_random_sample1=[int])

    def test_numpy_random_sample2(self):
        """ Check numpy random_sample with shape argument. """
        self.run_test("""
            def numpy_random_sample2(n):
                from numpy.random import random_sample
                from numpy import sum
                s = sum(random_sample((n, n)))
                return (abs(s / (n * n) - .5) < .05)""",
                      10 ** 3, numpy_random_sample2=[int])

    ###########################################################################
    #Tests for numpy.random.ranf
    ###########################################################################

    def test_numpy_ranf0(self):
        """ Check numpy ranf without arguments. """
        self.run_test("""
            def numpy_ranf0(n):
                from numpy.random import ranf
                from numpy import mean
                s = [ranf() for x in range(n)]
                return (abs(mean(s) - .5) < .05)""",
                      10 ** 5, numpy_ranf0=[int])

    def test_numpy_ranf1(self):
        """ Check numpy ranf with size argument. """
        self.run_test("""
            def numpy_ranf1(n):
                from numpy.random import ranf
                from numpy import mean
                a = ranf(size=n)
                return (abs(mean(a) - .5) < .05)""",
                      10 ** 5, numpy_ranf1=[int])

    def test_numpy_ranf2(self):
        """ Check numpy ranf with shape argument. """
        self.run_test("""
            def numpy_ranf2(n):
                from numpy.random import ranf
                from numpy import mean
                a = ranf((n, n))
                return (abs(mean(a) - .5) < .05)""",
                      10 ** 3, numpy_ranf2=[int])

    ###########################################################################
    #Tests for numpy.random.sample
    ###########################################################################

    def test_numpy_sample0(self):
        """ Check numpy sample without arguments. """
        self.run_test("""
            def numpy_sample0(n):
                from numpy.random import sample
                from numpy import mean
                s = [sample() for x in range(n)]
                return (abs(mean(s) - .5) < .05)""",
                      10 ** 5, numpy_sample0=[int])

    def test_numpy_sample1(self):
        """ Check numpy sample with size argument. """
        self.run_test("""
            def numpy_sample1(n):
                from numpy.random import sample
                from numpy import mean
                s = sample(size=n)
                return (abs(mean(s) - .5) < .05)""",
                      10 ** 5, numpy_sample1=[int])

    def test_numpy_sample2(self):
        """ Check numpy sample with shape argument. """
        self.run_test("""
            def numpy_sample2(n):
                from numpy.random import sample
                from numpy import mean
                s = sample((n, n))
                return (abs(mean(s) - .5) < .05)""",
                      10 ** 3, numpy_sample2=[int])

    ###########################################################################
    #Tests for numpy.random.rand
    ###########################################################################

    def test_numpy_rand0(self):
        """ Check numpy rand without arguments. """
        self.run_test("""
            def numpy_rand0(n):
                from numpy.random import rand
                s = sum(rand() for x in range(n))
                return (abs(s / n - .5) < .05)""",
                      10 ** 5, numpy_rand0=[int])

    def test_numpy_rand1(self):
        """ Check numpy rand with multiple arguments. """
        self.run_test("""
            def numpy_rand1(n):
                from numpy.random import rand
                from numpy import sum
                s = sum(rand(n, n))
                return (abs(s / (n * n) - .5) < .05)""",
                      10 ** 3, numpy_rand1=[int])

    ###########################################################################
    #Tests for numpy.random.binomial
    ###########################################################################

    def test_numpy_binomial0(self):
        code = """
        def numpy_binomial0(n, p, size):
         from numpy.random import binomial
         from numpy import var
         a = [binomial(n, p) for x in range(size)]
         return (abs(float(sum(a))/size - n * p) < .05 and abs(var(a) - n*p*(1-p)) < .05)
        """
        self.run_test(code, 10., .2, 10**5, numpy_binomial0=[float, float, int])

    def test_numpy_binomial1(self):
        code = """
        def numpy_binomial1(n, p, size):
         from numpy.random import binomial
         from numpy import var
         a=binomial(n, p, size)
         return (abs(float(sum(a))/size - n * p) < .05 and abs(var(a) - n*p*(1-p)) < .05)
        """
        self.run_test(code, 7., .2, 10**5, numpy_binomial1=[float, float, int])

    def test_numpy_binomial2(self):
        code = """
        def numpy_binomial2(n, p, size):
         from numpy.random import binomial
         from numpy import sum, var
         a=binomial(n, p, (size, size))
         return (abs(float(sum(a))/(size*size) - n * p) < .05 and abs(var(a) - n*p*(1-p)) < .05)
        """
        self.run_test(code, 9., .2, 10**3, numpy_binomial2=[float, float, int])

    def test_numpy_binomial_exception(self):
        code = """
        def numpy_binomial_exception():
            from numpy.random import binomial
            c = 0;
            try : binomial(-1, 0.1) 
            except ValueError : c += 1
            try : binomial(1, -1)
            except ValueError : c += 1
            try : binomial(1, 10)
            except ValueError : c += 1
            return c
        """
        self.run_test(code, numpy_binomial_exception=[])

    ###########################################################################
    #Tests for numpy.random.standard_normal
    ###########################################################################

    def test_numpy_standard_normal0(self):
        """ Check standard_normal without argument with mean and variance. """
        code = """
        def numpy_standard_normal0(size):
            from numpy.random import standard_normal
            from numpy import var, mean
            a = [standard_normal() for x in range(size)]
            print(mean(a))
            return (abs(mean(a)) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_standard_normal0=[int])

    def test_numpy_standard_normal1(self):
        """ Check standard_normal with size argument with mean and variance."""
        code = """
        def numpy_standard_normal1(size):
            from numpy.random import standard_normal
            from numpy import var, mean
            a = standard_normal(size)
            print(mean(a))
            return (abs(mean(a)) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_standard_normal1=[int])

    def test_numpy_standard_normal2(self):
        """Check standard_normal with shape argument with mean and variance."""
        code = """
        def numpy_standard_normal2(size):
            from numpy.random import standard_normal
            from numpy import mean, var
            a = standard_normal((size, size))
            print(mean(a))
            return (abs(mean(a)) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_standard_normal2=[int])

    ###########################################################################
    #Tests for numpy.random.normal
    ###########################################################################

    def test_numpy_normal0(self):
        """ Check normal without argument with mean and variance. """
        code = """
        def numpy_normal0(size):
            from numpy.random import normal
            from numpy import var, mean
            a = [normal() for x in range(size)]
            print(mean(a))
            return (abs(mean(a)) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_normal0=[int])

    def test_numpy_normal0a(self):
        """ Check normal with 1 argument with mean and variance. """
        code = """
        def numpy_normal0a(size):
            from numpy.random import normal
            from numpy import var, mean
            a = [normal(3.) for x in range(size)]
            print(mean(a))
            return (abs(mean(a)-3) < 0.05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_normal0a=[int])

    def test_numpy_normal0b(self):
        """ Check normal with 2 argument with mean and variance. """
        code = """
        def numpy_normal0b(size):
            from numpy.random import normal
            from numpy import var, mean, sqrt
            mu, sigma = 0, 0.1
            a = normal(mu, sigma, size)
            print(mean(a))
            return (abs(mu - mean(a)) < 0.05 and abs(sigma - sqrt(var(a,ddof=1))) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_normal0b=[int])



    def test_numpy_normal1(self):
        """ Check normal with size argument with mean and variance."""
        code = """
        def numpy_normal1(size):
            from numpy.random import normal
            from numpy import var, mean
            a = normal(size=size)
            print(mean(a))
            return (abs(mean(a)) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_normal1=[int])

    def test_numpy_normal2(self):
        """Check normal with shape argument with mean and variance."""
        code = """
        def numpy_normal2(size):
            from numpy.random import normal
            from numpy import mean, var
            a = normal(size=(size, size))
            print(mean(a))
            return (abs(mean(a)) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_normal2=[int])

    ###########################################################################
    #Tests for numpy.random.poisson
    ###########################################################################

    def test_numpy_poisson0(self):
        """ Check poisson without argument with mean and variance. """
        code = """
        def numpy_poisson0(size):
            from numpy.random import poisson
            from numpy import var, mean
            a = [poisson() for x in range(size)]
            print(mean(a))
            return (abs(mean(a)-1) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_poisson0=[int])

    def test_numpy_poisson0a(self):
        """ Check poisson with 1 argument with mean and variance. """
        code = """
        def numpy_poisson0a(size):
            from numpy.random import poisson
            from numpy import var, mean
            a = [poisson(3.) for x in range(size)]
            print(mean(a))
            return (abs(mean(a)-3) < .05 and abs(var(a) - 3) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_poisson0a=[int])

    def test_numpy_poisson0b(self):
        """ Check poisson with 2 argument with mean and variance. """
        code = """
        def numpy_poisson0b(size):
            from numpy.random import poisson
            from numpy import var, mean, sqrt
            lam = 10
            a = poisson(lam, size)
            print(mean(a))
            return (abs(mean(a)-lam) < 0.05 and abs(sqrt(lam) - sqrt(var(a,ddof=1))) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_poisson0b=[int])



    def test_numpy_poisson1(self):
        """ Check poisson with size argument with mean and variance."""
        code = """
        def numpy_poisson1(size):
            from numpy.random import poisson
            from numpy import var, mean
            a = poisson(size=size)
            print(mean(a))
            return (abs(mean(a)-1) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_poisson1=[int])

    def test_numpy_poisson2(self):
        """Check poisson with shape argument with mean and variance."""
        code = """
        def numpy_poisson2(size):
            from numpy.random import poisson
            from numpy import mean, var
            a = poisson(size=(size, size))
            print(mean(a))
            return (abs(mean(a)-1) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_poisson2=[int])

    ###########################################################################
    #Tests for numpy.random.randn
    ###########################################################################

    def test_numpy_randn0(self):
        """ Check numpy randn without arguments. """
        self.run_test("""
            def numpy_randn0(n):
                from numpy.random import randn
                from numpy import mean, var
                a = [randn() for x in range(n)]
                return (abs(mean(a)) < .05 and abs(var(a) - 1) < .05)""",
                      10 ** 5, numpy_randn0=[int])

    def test_numpy_randn1(self):
        """ Check numpy randn with multiple arguments. """
        self.run_test("""
            def numpy_randn1(n):
                from numpy.random import randn
                from numpy import mean, var
                a = randn(n, n)
                return (abs(mean(a)) < .05 and abs(var(a) - 1) < .05)""",
                      10 ** 3, numpy_randn1=[int])

    ###########################################################################
    #Tests for numpy.random.randint
    ###########################################################################

    def test_numpy_randint0(self):
        """ Check numpy randint with one argument. """
        self.run_test("""
            def numpy_randint0(n):
                from numpy.random import randint
                from numpy import mean, var
                a = [randint(11) for x in range(n)]
                return (abs(mean(a) - 5) < .05)""",
                      10 ** 5, numpy_randint0=[int])

    def test_numpy_randint1(self):
        """ Check numpy randint with min/max argument. """
        self.run_test("""
            def numpy_randint1(n):
                from numpy.random import randint
                from numpy import mean, var
                a = [randint(10, 21) for x in range(n)]
                return (abs(mean(a) - 15) < .05)""",
                      10 ** 5, numpy_randint1=[int])

    def test_numpy_randint2(self):
        """ Check numpy randint with size argument. """
        self.run_test("""
            def numpy_randint2(n):
                from numpy.random import randint
                from numpy import mean, var
                a = randint(10, 21, n)
                return (abs(mean(a) - 15) < .05)""",
                      10 ** 5, numpy_randint2=[int])

    def test_numpy_randint3(self):
        """ Check numpy randint with shape argument. """
        self.run_test("""
            def numpy_randint3(n):
                from numpy.random import randint
                from numpy import mean, var
                a = randint(10, 21, (n, n))
                return (abs(mean(a) - 15) < .05)""",
                      10 ** 3, numpy_randint3=[int])

    def test_numpy_randint4(self):
        """ Check numpy randint with shape argument. """
        self.run_test("""
            def numpy_randint4(n):
                from numpy.random import randint
                from numpy import mean, var
                a = randint(10, size=n)
                return (abs(mean(a) - 5) < .05)""",
                      10 ** 3, numpy_randint4=[int])

    ###########################################################################
    #Tests for numpy.random.random_integer
    ###########################################################################

    def test_numpy_random_integers0(self):
        """ Check numpy random_integers with one argument. """
        self.run_test("""
            def numpy_random_integers0(n):
                from numpy.random import random_integers
                from numpy import mean, var
                a = [random_integers(9) for x in range(n)]
                return all(0<=r<=9 for r in a)""",
                      3, numpy_random_integers0=[int])

    def test_numpy_random_integers1(self):
        """ Check numpy random_integers with min/max argument. """
        self.run_test("""
            def numpy_random_integers1(n):
                from numpy.random import random_integers
                from numpy import mean, var
                a = [random_integers(10, 20) for x in range(n)]
                return all(10<=r<=20 for r in a)""",
                      3, numpy_random_integers1=[int])

    def test_numpy_random_integers2(self):
        """ Check numpy random_integers with size argument. """
        self.run_test("""
            def numpy_random_integers2(n):
                from numpy.random import random_integers
                from numpy import mean, var
                a = random_integers(10, 20, n)
                return all(10<=r<=20 for r in a)""",
                      3, numpy_random_integers2=[int])

    def test_numpy_random_integers3(self):
        """ Check numpy random_integers with shape argument. """
        self.run_test("""
            def numpy_random_integers3(n):
                from numpy.random import random_integers
                from numpy import mean, var
                a = random_integers(10, 20, (n, n))
                return all(10<=r<=20 for r in a.flat)""",
                      3, numpy_random_integers3=[int])

    ###########################################################################
    #Tests for numpy.random.choice
    ###########################################################################

    def test_numpy_random_choice0(self):
        """ Check numpy.random.choice with one int argument. """
        self.run_test("""
            def numpy_random_choice0(n):
                from numpy.random import choice
                from numpy import mean, var
                a = [choice(11) for _ in range(n)]
                return (abs(mean(a) - 5) < .05)""",
                      10 ** 5, numpy_random_choice0=[int])

    def test_numpy_random_choice1(self):
        """ Check numpy.random.choice with one ndarray argument. """
        self.run_test("""
            def numpy_random_choice1(n):
                from numpy.random import choice
                from numpy import mean, var, arange
                a = [choice(arange(11)) for _ in range(n)]
                return (abs(mean(a) - 5) < .05)""",
                      10 ** 5, numpy_random_choice1=[int])

    def test_numpy_random_choice2(self):
        """ Check numpy.random.choice with one numpy_expr argument. """
        self.run_test("""
            def numpy_random_choice2(n, x):
                from numpy.random import choice
                from numpy import mean, var, arange
                a = [choice(arange(11) + n) for _ in range(n)]
                return (abs(mean(a) - (5 + x)) < .05)""",
                      10 ** 5, 1, numpy_random_choice2=[int, int])

    def test_numpy_random_choice3(self):
        """ Check numpy.random.choice with int and int argument. """
        self.run_test("""
            def numpy_random_choice3(n):
                from numpy.random import choice
                from numpy import mean, var, arange
                a = choice(11, n)
                return (abs(mean(a) - 5) < .05)""",
                      10 ** 5, numpy_random_choice3=[int])

    def test_numpy_random_choice4(self):
        """ Check numpy.random.choice with int and tuple argument. """
        self.run_test("""
            def numpy_random_choice4(n):
                from numpy.random import choice
                from numpy import mean, var, arange
                a = choice(11, (n, n))
                return (abs(mean(a) - 5) < .05)""",
                      10 ** 3, numpy_random_choice4=[int])

    def test_numpy_random_choice5(self):
        """ Check numpy.random.choice with int, tuple and proba argument. """
        self.run_test("""
            def numpy_random_choice5(n):
                from numpy.random import choice
                from numpy import mean, var, arange
                a = choice(5, (n, n), True, [0.3, 0.3, 0.2, 0.1, 0.1])
                return (abs(mean(a) - 1.4) < .05)""",
                      10 ** 3, numpy_random_choice5=[int])

    def test_numpy_random_choice6(self):
        """ Check numpy.random.choice with int, int and proba argument. """
        self.run_test("""
            def numpy_random_choice6(n):
                from numpy.random import choice
                from numpy import mean, var, arange
                a = choice(5, n, True, [0.3, 0.3, 0.2, 0.1, 0.1])
                return (abs(mean(a) - 1.4) < .05)""",
                      10 ** 5, numpy_random_choice6=[int])

    def test_numpy_random_choice7(self):
        """ Check numpy.random.choice with ndarray and int argument. """
        self.run_test("""
            def numpy_random_choice7(n):
                from numpy.random import choice
                from numpy import mean, var, arange
                a = choice(arange(11), n)
                return (abs(mean(a) - 5) < .05)""",
                      10 ** 5, numpy_random_choice7=[int])

    def test_numpy_random_choice8(self):
        """ Check numpy.random.choice with ndarray and tuple argument. """
        self.run_test("""
            def numpy_random_choice8(n):
                from numpy.random import choice
                from numpy import mean, var, arange
                a = choice(arange(11), (n, n))
                return (abs(mean(a) - 5) < .05)""",
                      10 ** 3, numpy_random_choice8=[int])

    def test_numpy_random_choice9(self):
        """Check numpy.random.choice with ndarray, tuple and proba argument."""
        self.run_test("""
            def numpy_random_choice9(n):
                from numpy.random import choice
                from numpy import mean, var, arange
                a = choice(arange(5), (n, n), True, [0.3, 0.3, 0.2, 0.1, 0.1])
                return (abs(mean(a) - 1.4) < .05)""",
                      10 ** 3, numpy_random_choice9=[int])

    def test_numpy_random_choice10(self):
        """ Check numpy.random.choice with ndarray, int and proba argument. """
        self.run_test("""
            def numpy_random_choice10(n):
                from numpy.random import choice
                from numpy import mean, var, arange
                a = choice(arange(5), n, True, [0.3, 0.3, 0.2, 0.1, 0.1])
                return (abs(mean(a) - 1.4) < .05)""",
                      10 ** 5, numpy_random_choice10=[int])

    ###########################################################################
    #Tests for numpy.random.bytes
    ###########################################################################

    def test_numpy_random_bytes1(self):
        """ Check numpy.random.bytes. """
        self.run_test("""
            def numpy_random_bytes1(n):
                from numpy.random import bytes
                from numpy import mean, fromstring, uint8, asarray
                a = bytes(n)
                return (abs(mean(asarray(fromstring(a, uint8), dtype=float)) - 127.5) < .05)""",
                      10 ** 8, numpy_random_bytes1=[int])

    ###########################################################################
    #Tests for numpy.random.exponential
    ###########################################################################

    def test_numpy_exponential0(self):
        """ Check exponential without argument with mean and variance. """
        code = """
        def numpy_exponential0(size):
            from numpy.random import exponential
            from numpy import var, mean
            a = [exponential() for x in range(size)]
            return (abs(mean(a) -1) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_exponential0=[int])

    def test_numpy_exponential0a(self):
        """ Check exponential with 1 argument with mean and variance. """
        code = """
        def numpy_exponential0a(size):
            from numpy.random import exponential
            from numpy import var, mean
            scale = 2.
            a = [exponential(scale) for x in range(size)]
            return (abs(mean(a) - scale) < 0.05 and abs(var(a) - scale**2) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_exponential0a=[int])

    def test_numpy_exponential0b(self):
        """ Check exponential with 2 argument with mean and variance. """
        code = """
        def numpy_exponential0b(size):
            from numpy.random import exponential
            from numpy import var, mean, sqrt
            scale = 2
            a = exponential(scale, size)
            return (abs(mean(a) - scale) < 0.05 and abs(var(a,ddof=1) - scale**2 ) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_exponential0b=[int])

    def test_numpy_exponential1(self):
        """ Check exponential with size argument with mean and variance."""
        code = """
        def numpy_exponential1(size):
            from numpy.random import exponential
            from numpy import var, mean
            a = exponential(size=size)
            return (abs(mean(a) -1 )< .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_exponential1=[int])

    def test_numpy_exponential2(self):
        """Check exponential with shape argument with mean and variance."""
        code = """
        def numpy_exponential2(size):
            from numpy.random import exponential
            from numpy import mean, var
            a = exponential(size=(size, size))
            return (abs(mean(a)) -1 < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_exponential2=[int])

    ###########################################################################
    #Tests for numpy.random.chisquare
    ###########################################################################

    def test_numpy_chisquare0a(self):
        """ Check chisquare with 1 argument with mean and variance. """
        code = """
        def numpy_chisquare0a(size):
            from numpy.random import chisquare
            from numpy import var, mean
            df = 3.
            a = [chisquare(df) for x in range(size)]
            return (abs(mean(a) - df) < 0.05 and abs(var(a) - 2*df) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_chisquare0a=[int])

    def test_numpy_chisquare0b(self):
        """ Check chisquare with 2 argument with mean and variance. """
        code = """
        def numpy_chisquare0b(size):
            from numpy.random import chisquare
            from numpy import var, mean, sqrt
            df = 2
            a = chisquare(df, size)
            return (abs(mean(a) - df) < 0.05 and abs(var(a) - df*2 ) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_chisquare0b=[int])

    def test_numpy_chisquare2(self):
        """Check chisquare with shape argument with mean and variance."""
        code = """
        def numpy_chisquare2(size):
            from numpy.random import chisquare
            from numpy import mean, var
            df = 1
            a = chisquare(df, size=(size, size))
            return (abs(mean(a)) - df < .05 and abs(var(a) - 2*df) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_chisquare2=[int])

    ###########################################################################
    #Tests for numpy.random.gamma
    ###########################################################################

    def test_numpy_gamma0a(self):
        """ Check gamma with 1 argument with mean and variance. """
        code = """
        def numpy_gamma0a(size):
            from numpy.random import gamma
            from numpy import var, mean
            shape = 1
            a = [gamma(3.) for x in range(size)]
            return (abs(mean(a)- shape) < 0.1 and abs(var(a) - shape) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_gamma0a=[int])

    def test_numpy_gamma0b(self):
        """ Check gamma with 2 argument with mean and variance. """
        code = """
        def numpy_gamma0b(size):
            from numpy.random import gamma
            from numpy import var, mean, sqrt
            shape, scale = 1,2
            a = gamma(shape, scale, size)
            return (abs(mean(a) - shape*scale) < 0.05 and abs(var(a) - shape*scale**2) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_gamma0b=[int])

    def test_numpy_gamma2(self):
        """Check gamma with shape argument with mean and variance."""
        code = """
        def numpy_gamma2(size):
            from numpy.random import gamma
            from numpy import mean, var
            shape = 2
            a = gamma(shape = shape, size=(size, size))
            return (abs(mean(a) - shape) < .05 and abs(var(a) - shape) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_gamma2=[int])

    ###########################################################################
    #Tests for numpy.random.weibull
    ###########################################################################

    def test_numpy_weibull0a(self):
        """ Check weibull with 1 argument with mean and variance. """
        code = """
        def numpy_weibull0a(size):
            from numpy.random import weibull
            from numpy import var, mean
            pa = 3.
            a = [weibull(pa) for x in range(size)]
            return (abs(mean(a) - pa) < 0.05 and abs(var(a) - 2*pa) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_weibull0a=[int])

    def test_numpy_weibull0b(self):
        """ Check weibull with 2 argument with mean and variance. """
        code = """
        def numpy_weibull0b(size):
            from numpy.random import weibull
            from numpy import var, mean, sqrt
            pa = 2
            a = weibull(pa, size)
            return (abs(mean(a) - pa) < 0.05 and abs(var(a) - pa*2 ) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_weibull0b=[int])

    def test_numpy_weibull2(self):
        """Check weibull with shape argument with mean and variance."""
        code = """
        def numpy_weibull2(size):
            from numpy.random import weibull
            from numpy import mean, var
            pa = 1
            a = weibull(pa, size=(size, size))
            return (abs(mean(a)) - pa < .05 and abs(var(a) - 2*pa) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_weibull2=[int])

    ###########################################################################
    #Tests for numpy.random.lognormal
    ###########################################################################

    def test_numpy_lognormal0(self):
        """ Check lognormal without argument with mean and variance. """
        code = """
        def numpy_lognormal0(size):
            from numpy.random import lognormal
            from numpy import var, mean, e
            a = [lognormal() for x in range(size)]
            m = 0
            s = 1/2
            rmean = e**(m+(s**2/2))  
            rvar = (e**(s**2) - 1)*e**(2*m+s**2) 
            return (abs(mean(a) - rmean) < .1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_lognormal0=[int])

    def test_numpy_lognormal0a(self):
        """ Check lognormal with 1 argument with mean and variance. """
        code = """
        def numpy_lognormal0a(size):
            from numpy.random import lognormal
            from numpy import var, mean, e
            m = 0
            s = 1/5
            a = [lognormal(m) for x in range(size)]
            rmean = e**(m+(s**2/2))  
            rvar = (e**(s**2) - 1)*e**(2*m+s**2) 
            return (abs(mean(a)- rmean) < 0.1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_lognormal0a=[int])

    def test_numpy_lognormal0b(self):
        """ Check lognormal with 2 argument with mean and variance. """
        code = """
        def numpy_lognormal0b(size):
            from numpy.random import lognormal
            from numpy import var, mean, e
            m = 1
            s = 1/8
            a = lognormal(m, s, size)
            rmean = e**(m+(s**2/2))  
            rvar = (e**(s**2) - 1)*e**(2*m+s**2)
            return (abs(mean(a) - rmean) < 0.1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_lognormal0b=[int])

    def test_numpy_lognormal1(self):
        """ Check lognormal with size argument with mean and variance."""
        code = """
        def numpy_lognormal1(size):
            from numpy.random import lognormal
            from numpy import var, mean, e
            m = 0
            s = 1/4
            rmean = e**(m+(s**2/2))  
            rvar = (e**(s**2) - 1)*e**(2*m+s**2) 
            a = lognormal(size=size)
            return (abs(mean(a) - rmean) < .1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_lognormal1=[int])

    def test_numpy_lognormal2(self):
        """Check lognormal with shape argument with mean and variance."""
        code = """
        def numpy_lognormal2(size):
            from numpy.random import lognormal
            from numpy import mean, var, e
            m = 2
            s = 1/2
            rmean = e**(m+(s**2/2))  
            rvar = (e**(s**2) - 1)*e**(2*m+s**2) 
            a = lognormal(size=(size, size))
            return (abs(mean(a) - rmean) < .1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 3, numpy_lognormal2=[int])

    ###########################################################################
    #Tests for numpy.random.geometric
    ###########################################################################

    def test_numpy_geometric0a(self):
        """ Check geometric with 1 argument with mean and variance. """
        code = """
        def numpy_geometric0a(size):
            from numpy.random import geometric
            from numpy import var, mean
            a = [geometric(0.6) for x in range(size)]
            return (abs(mean(a)- 2) < .05 and abs(var(a) - 3) < 1/8)
        """
        self.run_test(code, 10 ** 6, numpy_geometric0a=[int])

    def test_numpy_geometric0b(self):
        """ Check geometric with 2 argument with mean and variance. """
        code = """
        def numpy_geometric0b(size):
            from numpy.random import geometric
            from numpy import var, mean, sqrt
            p = 0.25
            a = geometric(p, size)
            return (abs(mean(a)- 4) < 0.05 and abs(sqrt(p) - sqrt(var(a,ddof=1))) < 1/64)
        """
        self.run_test(code, 10 ** 6, numpy_geometric0b=[int])




    def test_numpy_geometric2(self):
        """Check geometric with shape argument with mean and variance."""
        code = """
        def numpy_geometric2(size):
            from numpy.random import geometric
            from numpy import mean, var
            p = 0.5
            a = geometric(p, size=(size, size))
            return (abs(mean(a)-2) < .05 and abs(var(a) - 1) < 1/8)
        """
        self.run_test(code, 10 ** 3, numpy_geometric2=[int])


    ###########################################################################
    #Tests for numpy.random.pareto
    ###########################################################################

    def test_numpy_pareto0a(self):
        """ Check pareto with 1 argument with mean and variance. """
        code = """
        def numpy_pareto0a(size):
            from numpy.random import pareto
            from numpy import var, mean
            alpha = 10
            rvar = alpha/((alpha-1)**2*(alpha-2))
            a = [pareto(alpha) for x in range(size)]
            return (abs(mean(a)- 0.5) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_pareto0a=[int])

    def test_numpy_pareto0b(self):
        """ Check pareto with 2 argument with mean and variance. """
        code = """
        def numpy_pareto0b(size):
            from numpy.random import pareto
            from numpy import var, mean, sqrt
            alpha = 6
            rvar = alpha/((alpha-1)**2*(alpha-2))
            a = pareto(alpha, size)
            return (abs(mean(a)- 0.5) < 0.05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_pareto0b=[int])

    def test_numpy_pareto2(self):
        """Check pareto with shape argument with mean and variance."""
        code = """
        def numpy_pareto2(size):
            from numpy.random import pareto
            from numpy import mean, var
            alpha = 5
            rvar = alpha/((alpha-1)**2*(alpha-2))
            a = pareto(alpha, size=(size, size))
            return (abs(mean(a)- 0.5) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_pareto2=[int])

    ###########################################################################
        #Tests for numpy.random.power
    ###########################################################################

    def test_numpy_power0a(self):
        """ Check power with 1 argument with mean and variance. """
        code = """
        def numpy_power0a(size):
            from numpy.random import power
            from numpy import var, mean
            alpha = 1
            rmean = alpha / (alpha + 1)
            rvar = alpha/((alpha+1)**2*(alpha+2))
            a = [power(alpha) for x in range(size)]
            return (abs(mean(a)- rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_power0a=[int])

    def test_numpy_power0b(self):
        """ Check power with 2 argument with mean and variance. """
        code = """
        def numpy_power0b(size):
            from numpy.random import power
            from numpy import var, mean, sqrt
            alpha = 1
            rmean = alpha / (alpha + 1)
            rvar = alpha/((alpha+1)**2*(alpha+2))
            a = power(alpha, size)
            return (abs(mean(a)- rmean) < 0.05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_power0b=[int])

    def test_numpy_power2(self):
        """Check power with shape argument with mean and variance."""
        code = """
        def numpy_power2(size):
            from numpy.random import power
            from numpy import mean, var
            alpha = 1
            rmean = alpha / (alpha + 1)
            rvar = alpha/((alpha+1)**2*(alpha+2))
            a = power(alpha, size=(size, size))
            return (abs(mean(a)- rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_power2=[int])

    ###########################################################################
    #Tests for numpy.random.rayleigh
    ###########################################################################

    def test_numpy_rayleigh0(self):
        """ Check rayleigh without argument with mean and variance. """
        code = """
        def numpy_rayleigh0(size):
            from numpy.random import rayleigh
            from numpy import var, mean, sqrt, pi
            a = [rayleigh() for x in range(size)]
            s = 2
            rmean = s*sqrt(pi/2)
            rvar = ((4-pi)/2)*s**2
            return (abs(mean(a)-rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_rayleigh0=[int])

    def test_numpy_rayleigh0a(self):
        """ Check rayleigh with 1 argument with mean and variance. """
        code = """
        def numpy_rayleigh0a(size):
            from numpy.random import rayleigh
            from numpy import var, mean, sqrt, pi
            s = 2
            a = [rayleigh(s) for x in range(size)]
            rmean = s*sqrt(pi/2)
            rvar = ((4-pi)/2)*s**2
            return (abs(mean(a)-rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_rayleigh0a=[int])

    def test_numpy_rayleigh0b(self):
        """ Check rayleigh with 2 argument with mean and variance. """
        code = """
        def numpy_rayleigh0b(size):
            from numpy.random import rayleigh
            from numpy import var, mean, sqrt, pi
            s = 2
            a = rayleigh(s, size)
            rmean = s*sqrt(pi/2)
            rvar = ((4-pi)/2)*s**2
            return (abs(mean(a)-rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_rayleigh0b=[int])

    def test_numpy_rayleigh1(self):
        """ Check rayleigh with size argument with mean and variance."""
        code = """
        def numpy_rayleigh1(size):
            from numpy.random import rayleigh
            from numpy import var, mean, sqrt, pi
            a = rayleigh(size=size)
            s = 2
            rmean = s*sqrt(pi/2)
            rvar = ((4-pi)/2)*s**2
            return (abs(mean(a)-rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_rayleigh1=[int])

    def test_numpy_rayleigh2(self):
        """Check rayleigh with shape argument with mean and variance."""
        code = """
        def numpy_rayleigh2(size):
            from numpy.random import rayleigh
            from numpy import mean, var, sqrt, pi
            a = rayleigh(size=(size, size))
            s = 2
            rmean = s*sqrt(pi/2)
            rvar = ((4-pi)/2)*s**2
            return (abs(mean(a)-rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_rayleigh2=[int])

    ###########################################################################
    #Tests for numpy.random.f
    ###########################################################################

    def test_numpy_f0a(self):
        """ Check f with 2 argument with mean and variance. """
        code = """
        def numpy_f0a(size):
            from numpy.random import f
            from numpy import var, mean
            dfnum = 50
            dfden = 50
            rmean =  dfden / (dfden - 2)
            rvar = (2 * dfden**2 *( dfnum + dfden -2))/(dfnum * (dfden -2)**2 * (dfden -4))
            a = [f(dfnum, dfden) for x in range(size)]
            return (abs(mean(a)- rmean) < 0.1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_f0a=[int])

    def test_numpy_f0b(self):
        """ Check f with 2 argument with mean and variance. """
        code = """
        def numpy_f0b(size):
            from numpy.random import f
            from numpy import var, mean
            dfnum = 50
            dfden = 50
            rmean =  dfden / (dfden - 2)
            rvar = (2 * dfden**2 *( dfnum + dfden -2))/(dfnum * (dfden -2)**2 * (dfden -4))
            a = f(dfnum, dfden, size)
            return (abs(mean(a) - rmean) < 0.1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_f0b=[int])

    def test_numpy_f2(self):
        """Check f with shape argument with mean and variance."""
        code = """
        def numpy_f2(size):
            from numpy.random import f
            from numpy import mean, var
            dfnum = 50
            dfden = 50
            rmean =  dfden / (dfden - 2)
            rvar = (2 * dfden**2 *( dfnum + dfden -2))/(dfnum * (dfden -2)**2 * (dfden -4))
            a = f(dfnum, dfden, size=(size, size))
            return (abs(mean(a) - rmean) < .1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 3, numpy_f2=[int])

    ###########################################################################
    #Tests for numpy.random.negative_binomial
    ###########################################################################

    def test_numpy_negative_binomial0a(self):
        """ Check negative_binomial with 1 argument with mean and variance. """
        code = """
        def numpy_negative_binomial0a(size):
            from numpy.random import negative_binomial
            from numpy import var, mean
            n = 1
            p = 1
            rmean = (n*(1-p))/p
            rvar = (n*(1-p))/p**2
            a = [negative_binomial(n,p) for x in range(size)]
            return (abs(mean(a)- rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_negative_binomial0a=[int])

    def test_numpy_negative_binomial0b(self):
        """ Check negative_binomial with 2 argument with mean and variance. """
        code = """
        def numpy_negative_binomial0b(size):
            from numpy.random import negative_binomial
            from numpy import var, mean, sqrt
            n = 1
            p = 1
            rmean = (n*(1-p))/p
            rvar = (n*(1-p))/p**2
            a = negative_binomial(n, p, size)
            return (abs(mean(a)- rmean) < 0.05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_negative_binomial0b=[int])

    def test_numpy_negative_binomial2(self):
        """Check negative_binomial with shape argument with mean and variance."""
        code = """
        def numpy_negative_binomial2(size):
            from numpy.random import negative_binomial
            from numpy import mean, var
            n = 1
            p = 1
            rmean = (n*(1-p))/p
            rvar = (n*(1-p))/p**2
            a = negative_binomial(n, p , size=(size, size))
            return (abs(mean(a)- rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_negative_binomial2=[int])

    ###########################################################################
    #Tests for numpy.random.standard_exponential
    ###########################################################################

    def test_numpy_standard_exponential0(self):
        """ Check standard_exponential without argument with mean and variance. """
        code = """
        def numpy_standard_exponential0(size):
            from numpy.random import standard_exponential
            from numpy import var, mean
            a = [standard_exponential() for x in range(size)]
            return (abs(mean(a) - 1) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_standard_exponential0=[int])

    def test_numpy_standard_exponential1(self):
        """ Check standard_exponential with size argument with mean and variance."""
        code = """
        def numpy_standard_exponential1(size):
            from numpy.random import standard_exponential
            from numpy import var, mean
            a = standard_exponential(size)
            return (abs(mean(a) - 1) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_standard_exponential1=[int])

    def test_numpy_standard_exponential2(self):
        """Check standard_exponential with shape argument with mean and variance."""
        code = """
        def numpy_standard_exponential2(size):
            from numpy.random import standard_exponential
            from numpy import mean, var
            a = standard_exponential((size, size))
            return (abs(mean(a) - 1) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_standard_exponential2=[int])

    ###########################################################################
    #Tests for numpy.random.standard_gamma
    ###########################################################################

    def test_numpy_standard_gamma0(self):
        """ Check standard_gamma without argument with mean and variance. """
        code = """
        def numpy_standard_gamma0(size):
            from numpy.random import standard_gamma
            from numpy import var, mean
            a = [standard_gamma(1) for x in range(size)]
            return (abs(mean(a) - 1) < .05 and abs(var(a) - 1) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_standard_gamma0=[int])

    def test_numpy_standard_gamma1(self):
        """ Check standard_gamma with size argument with mean and variance."""
        code = """
        def numpy_standard_gamma1(size):
            from numpy.random import standard_gamma
            from numpy import var, mean
            a = standard_gamma(2, size)
            return (abs(mean(a) - 2) < .05 and abs(var(a) - 2) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_standard_gamma1=[int])

    def test_numpy_standard_gamma2(self):
        """Check standard_gamma with shape argument with mean and variance."""
        code = """
        def numpy_standard_gamma2(size):
            from numpy.random import standard_gamma
            from numpy import mean, var
            a = standard_gamma(3, (size, size))
            return (abs(mean(a) - 3) < .05 and abs(var(a) - 3) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_standard_gamma2=[int])

    ###########################################################################
    #Tests for numpy.random.gumbel
    ###########################################################################

    def test_numpy_gumbel0(self):
        """ Check gumbel without argument with mean and variance. """
        code = """
        def numpy_gumbel0(size):
            from numpy.random import gumbel
            from numpy import var, mean, pi
            u = 0.
            rmean = u + 0.57721
            rvar = (pi**2/6)
            a = [gumbel() for x in range(size)]
            return (abs(mean(a) - rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_gumbel0=[int])

    def test_numpy_gumbel0a(self):
        """ Check gumbel with 1 argument with mean and variance. """
        code = """
        def numpy_gumbel0a(size):
            from numpy.random import gumbel
            from numpy import var, mean, pi
            u = 1
            rmean = u + 0.57721
            rvar = (pi**2/6)
            a = [gumbel(u) for x in range(size)]
            return (abs(mean(a) - rmean ) < 0.05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_gumbel0a=[int])

    def test_numpy_gumbel0b(self):
        """ Check gumbel with 2 argument with mean and variance. """
        code = """
        def numpy_gumbel0b(size):
            from numpy.random import gumbel
            from numpy import var, mean, pi
            u = 1.5
            s = 2
            rmean = u + 0.57721*s
            rvar = (pi**2/6)*s**2
            a = gumbel(u, s, size)
            return (abs(mean(a) - rmean) < 0.05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_gumbel0b=[int])

    def test_numpy_gumbel1(self):
        """ Check gumbel with size argument with mean and variance."""
        code = """
        def numpy_gumbel1(size):
            from numpy.random import gumbel
            from numpy import var, mean
            from numpy import var, mean, pi
            u = 0.
            s = 1
            rmean = u + 0.57721*s
            rvar = (pi**2/6)*s**2
            a = gumbel(size=size)
            return (abs(mean(a) - rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_gumbel1=[int])

    def test_numpy_gumbel2(self):
        """Check gumbel with shape argument with mean and variance."""
        code = """
        def numpy_gumbel2(size):
            from numpy.random import gumbel
            from numpy import mean, var
            from numpy import var, mean, pi
            u = 0
            s = 1
            rmean = u + 0.57721*s
            rvar = (pi**2/6)*s**2
            a = gumbel(size=(size, size))
            return (abs(mean(a) - rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_gumbel2=[int])

    ###########################################################################
    #Tests for numpy.random.logistic
    ###########################################################################

    def test_numpy_logistic0(self):
        """ Check logistic without argument with mean and variance. """
        code = """
        def numpy_logistic0(size):
            from numpy.random import logistic
            from numpy import var, mean, pi
            u = 0.
            rmean = u
            rvar = (pi**2/3)
            a = [logistic() for x in range(size)]
            return (abs(mean(a) - rmean) < .1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_logistic0=[int])

    def test_numpy_logistic0a(self):
        """ Check logistic with 1 argument with mean and variance. """
        code = """
        def numpy_logistic0a(size):
            from numpy.random import logistic
            from numpy import var, mean, pi
            u = 2
            rmean = u
            rvar = (pi**2/3)
            a = [logistic(u) for x in range(size)]
            return (abs(mean(a) - rmean ) < 0.1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_logistic0a=[int])

    def test_numpy_logistic0b(self):
        """ Check logistic with 2 argument with mean and variance. """
        code = """
        def numpy_logistic0b(size):
            from numpy.random import logistic
            from numpy import var, mean, pi
            u = 2.
            s = 2
            rmean = u
            rvar = ((s**2*pi**2)/3)
            a = logistic(u, s, size)
            return (abs(mean(a) - rmean) < 0.1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_logistic0b=[int])

    def test_numpy_logistic1(self):
        """ Check logistic with size argument with mean and variance."""
        code = """
        def numpy_logistic1(size):
            from numpy.random import logistic
            from numpy import var, mean
            from numpy import var, mean, pi
            u = 0.
            s = 1
            rmean = u
            rvar = ((s**2*pi**2)/3)
            a = logistic(size=size)
            return (abs(mean(a) - rmean) < .1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 6, numpy_logistic1=[int])

    def test_numpy_logistic2(self):
        """Check logistic with shape argument with mean and variance."""
        code = """
        def numpy_logistic2(size):
            from numpy.random import logistic
            from numpy import mean, var
            from numpy import var, mean, pi
            u = 0
            s = 1
            rmean = u
            rvar = ((s**2*pi**2)/3)
            a = logistic(size=(size, size))
            return (abs(mean(a) - rmean) < .1 and abs(var(a) - rvar) < .1)
        """
        self.run_test(code, 10 ** 3, numpy_logistic2=[int])

    ###########################################################################
    #Tests for numpy.random.laplace
    ###########################################################################

    def test_numpy_laplace0(self):
        """ Check laplace without argument with mean and variance. """
        code = """
        def numpy_laplace0(size):
            from numpy.random import laplace
            from numpy import var, mean, pi
            u = 0.
            s = 1
            rmean = u
            rvar = 2*s**2
            a = [laplace() for x in range(size)]
            return (abs(mean(a) - rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_laplace0=[int])

    def test_numpy_laplace0a(self):
        """ Check laplace with 1 argument with mean and variance. """
        code = """
        def numpy_laplace0a(size):
            from numpy.random import laplace
            from numpy import var, mean, pi
            u = 2
            s = 1
            rmean = u
            rvar = 2*s**2
            a = [laplace(u) for x in range(size)]
            return (abs(mean(a) - rmean ) < 0.05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_laplace0a=[int])

    def test_numpy_laplace0b(self):
        """ Check laplace with 2 argument with mean and variance. """
        code = """
        def numpy_laplace0b(size):
            from numpy.random import laplace
            from numpy import var, mean, pi
            u = 2.
            s = 2
            rmean = u
            rvar = 2*s**2
            a = laplace(u, s, size)
            return (abs(mean(a) - rmean) < 0.05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_laplace0b=[int])

    def test_numpy_laplace1(self):
        """ Check laplace with size argument with mean and variance."""
        code = """
        def numpy_laplace1(size):
            from numpy.random import laplace
            from numpy import var, mean
            from numpy import var, mean, pi
            u = 0.
            s = 1
            rmean = u
            rvar = 2*s**2
            a = laplace(size=size)
            return (abs(mean(a) - rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 6, numpy_laplace1=[int])

    def test_numpy_laplace2(self):
        """Check laplace with shape argument with mean and variance."""
        code = """
        def numpy_laplace2(size):
            from numpy.random import laplace
            from numpy import mean, var
            from numpy import var, mean, pi
            u = 0
            s = 1
            rmean = u
            rvar = 2*s**2
            a = laplace(size=(size, size))
            return (abs(mean(a) - rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_laplace2=[int])

    ###########################################################################
    #Tests for numpy.random.logseries
    ###########################################################################

    def test_numpy_logseries0(self):
        """ Check logseries without argument with mean and variance. """
        code = """
        def numpy_logseries0(size):
            from numpy.random import logseries
            from numpy import var, mean, log
            s = 0.5
            rmean = s / (log(1 - s)*(s - 1))
            rvar = -(s*(s+log(1-s)))/((s - 1)**2*(log(1-s))**2)      
            a = [logseries(s) for x in range(size)]
            return (abs(mean(a) - rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_logseries0=[int])

    def test_numpy_logseries1(self):
        """ Check logseries with size argument with mean and variance."""
        code = """
        def numpy_logseries1(size):
            from numpy.random import logseries
            from numpy import var, mean, log
            s = 0.25
            rmean = s / (log(1 - s)*(s - 1))
            rvar = -(s*(s+log(1-s)))/((s - 1)**2*(log(1-s))**2)             
            a = logseries(s, size)
            return (abs(mean(a) - rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 5, numpy_logseries1=[int])

    def test_numpy_logseries2(self):
        """Check logseries with shape argument with mean and variance."""
        code = """
        def numpy_logseries2(size):
            from numpy.random import logseries
            from numpy import mean, var, log
            s = 0.2
            rmean = s / (log(1 - s)*(s - 1))
            rvar = -(s*(s+log(1-s)))/((s - 1)**2*(log(1-s))**2)           
            a = logseries(s, (size, size))
            return (abs(mean(a) - rmean) < .05 and abs(var(a) - rvar) < .05)
        """
        self.run_test(code, 10 ** 3, numpy_logseries2=[int])

    ###########################################################################
    # Tests for numpy.random.uniform
    ###########################################################################

    def test_numpy_uniform_no_arg(self):
        """ Check logseries without argument with mean and variance. """
        code = """
        def numpy_uniform_no_arg(size):
            import numpy as np
            from numpy.random import uniform
            low, high = 0.0, 1.0
            a = np.array([uniform() for _ in range(size)])
            rmean = 0.5 * (low + high)
            rvar = (high - low) ** 2 / 12
            cond_mean = (a.mean() - rmean) / rmean < 0.05
            cond_var = (np.var(a) - rvar) / rvar < 0.05
            return cond_mean and cond_var
        """
        self.run_test(code, 4000, numpy_uniform_no_arg=[int])

    def test_numpy_uniform_size_int(self):
        """ Check logseries with arguments with mean and variance. """
        code = """
        def numpy_uniform_size_int(size):
            import numpy as np
            from numpy.random import uniform
            low, high = 0., 1234.
            rmean = 0.5 * (low + high)
            rvar = (high - low) ** 2 / 12
            a = uniform(low, high, size)
            cond_mean = (a.mean() - rmean) / rmean < 0.05
            cond_var = (np.var(a) - rvar) / rvar < 0.05
            return cond_mean and cond_var
        """
        self.run_test(code, 4000, numpy_uniform_size_int=[int])


    def test_numpy_uniform_size_tuple(self):
        """ Check logseries with arguments with mean and variance. """
        code = """
        def numpy_uniform_size_tuple(size):
            import numpy as np
            from numpy.random import uniform
            low, high = -987., 12345.
            rmean = 0.5 * (low + high)
            rvar = (high - low) ** 2 / 12
            a = uniform(low, high, (size, size))
            cond_mean = (a.mean() - rmean) / rmean < 0.05
            cond_var = (np.var(a) - rvar) / rvar < 0.05
            return cond_mean and cond_var
        """
        self.run_test(code, 70, numpy_uniform_size_tuple=[int])



#pythran export guerre(complex list, int, complex, float, int)
#runas guerre([complex(1,2),complex(3,4),complex(5,6),complex(7,8)],2,complex(5.6,4.3),-3.4,20)
#bench guerre([complex(1,2),complex(3,4),complex(5,6),complex(7,8)],2,complex(5.6,4.3),-3.4,400000)
def guerre(a,n,z,err,nter):
    az = [complex(0,0) for i in range(50)]
    azz = [complex(0,0) for i in range(50)]
    ci=complex(0.0,1.0)
    eps=1.0e-20
#  The coefficients of p'[z] and p''[z].
    for i in range(1,n+1):
        az[i-1]=float(i)*a[i]
    for i in range(1,n):
        azz[i-1]=float(i)*az[i]
    dz=err+1
    itera=0
    jter=0
    while abs(dz)>err and itera<nter:
        p=a[n-1]+a[n]*z
        for i in range(n-1,0,-1):
            p=a[i-1]+z*p
        if abs(p)<eps:
            return z
        pz=az[n-2]+az[n-1]*z
        for i in range(n-2,0,-1):
            pz=az[i-1]+z*pz
        pzz=azz[n-3]+azz[n-2]*z
        for i in range(n-3,0,-1):
            pzz=azz[i-1]+z*pzz
#  The Laguerre perturbation.
        f=pz/p
        g=(f**2)-pzz/p
        h= n*g#cmath.sqrt((float(n)-1.0)*(n*g-(f**2)))
        amp1=abs(f+h);
        amp2=abs(f-h);
        if amp1>amp2:
            dz=float(-n)/(f+h)
        else:
            dz=float(-n)/(f-h)
        itera=itera+1
#   Rotate by 90 degrees to avoid limit cycles.
        jter=jter+1
        if jter==10:
            jter=1
            dz=dz*ci
        z=z+dz
        if jter==100:
            raise RuntimeError("Laguerre method not converging")
    return z



#from: http://stackoverflow.com/questions/13815719/creating-grid-with-numpy-performance
#pythran export create_grid(float [])
#runas import numpy as np ; N = 1000 ; x = np.arange(0,1,1./N) ; create_grid(x)
#bench import numpy as np ; N = 1000 ; x = np.arange(0,1,1./N) ; create_grid(x)
import numpy as np
def create_grid(x):
    N = x.shape[0]
    z = np.zeros((N, N, 3))
    z[:,:,0] = x.reshape(-1,1)
    z[:,:,1] = x
    fast_grid = z.reshape(N*N, 3)
    return fast_grid



def omp_for_firstprivate():
    sum = 0
    sum0 = 12345
    sum1 = 0
    import omp
    LOOPCOUNT = 1000
    if 'omp parallel private(sum1)':
        'omp single'
        threadsnum = omp.get_num_threads()
        'omp for firstprivate(sum0)'
        for i in range(1, LOOPCOUNT+1):
            sum0+=i
            sum1 = sum0
        'omp critical'
        sum+=sum1
    known_sum = 12345* threadsnum+ (LOOPCOUNT * (LOOPCOUNT + 1)) / 2
    return sum == known_sum



def omp_taskwait():
    from time import sleep
    result1 = 0
    result2 = 0
    NUM_TASKS = 25
    array = [0 for _ in range(NUM_TASKS)]

    if 'omp parallel':
        if 'omp single':
            for i in range(NUM_TASKS):
                myi = i
                if 'omp task firstprivate(myi)':
                    sleep(0.01)
                    array[myi] = 1

            'omp taskwait'

            for i in range(NUM_TASKS):
                if array[i] != 1:
                    result1 += 1

            for i in range(NUM_TASKS):
                myi = i
                if 'omp task firstprivate(myi)':
                    array[myi] = 2

    for i in range(NUM_TASKS):
        if array[i] != 2:
            result2 += 1

    return result1 == 0 and result2 == 0



#from http://rosettacode.org/wiki/Reduced_row_echelon_form#Python
#pythran export ToReducedRowEchelonForm(int list list)
#runas mtx = [ [ 1, 2, -1, -4], [ 2, 3, -1, -11], [-2, 0, -3, 22],]; ToReducedRowEchelonForm(mtx)

def ToReducedRowEchelonForm( M):
    if not M: return
    lead = 0
    rowCount = len(M)
    columnCount = len(M[0])
    for r in range(rowCount):
        if lead >= columnCount:
            return
        i = r
        while M[i][lead] == 0:
            i += 1
            if i == rowCount:
                i = r
                lead += 1
                if columnCount == lead:
                    return
        M[i],M[r] = M[r],M[i]
        lv = M[r][lead]
        M[r] = [ mrx / lv for mrx in M[r]]
        for i in range(rowCount):
            if i != r:
                lv = M[i][lead]
                M[i] = [ iv - lv*rv for rv,iv in zip(M[r],M[i])]
        lead += 1



#pythran export arc_distance(float [:], float[], float[], float[])
#runas import numpy as np; arc_distance(np.array([12.4,0.5,-5.6,12.34,9.21]),np.array([-5.6,3.4,2.3,-23.31,12.6]),np.array([3.45,1.5,55.4,567.0,43.2]),np.array([56.1,3.4,1.34,-56.9,-3.4]))
#bench import numpy.random; N=5000000; a, b, c, d = numpy.random.rand(N), numpy.random.rand(N), numpy.random.rand(N), numpy.random.rand(N); arc_distance(a, b, c, d)

import numpy as np
def arc_distance(theta_1, phi_1,
                 theta_2, phi_2):
    """
    Calculates the pairwise arc distance between all points in vector a and b.
    """
    temp = np.sin((theta_2-theta_1)/2)**2+np.cos(theta_1)*np.cos(theta_2)*np.sin((phi_2-phi_1)/2)**2
    distance_matrix = 2 * (np.arctan2(np.sqrt(temp),np.sqrt(1-temp)))
    return distance_matrix



from pythran.tests import TestEnv
from pythran.typing import *

class TestCopperhead(TestEnv):

# from copperhead test suite
# https://github.com/copperhead

    def test_saxpy(self):
        self.run_test(
            "def saxpy(a, x, y): return list(map(lambda xi, yi: a * xi + yi, x, y))",
            1.5, [1,2,3], [0.,2.,4.],
            saxpy=[float, List[int], List[float]])

    def test_saxpy2(self):
        self.run_test(
            "def saxpy2(a, x, y): return [a*xi+yi for xi,yi in zip(x,y)]",
            1.5, [1,2,3], [0.,2.,4.],
            saxpy2=[float,List[int], List[float]])

    def test_saxpy3(self):
        code="""
def saxpy3(a, x, y):
    def triad(xi, yi): return a * xi + yi
    return list(map(triad, x, y))
"""
        self.run_test(
            code,
            1.5, [1,2,3], [0.,2.,4.],
            saxpy3=[float,List[int], List[float]])

    def test_saxpy4(self):
        code="""
def saxpy4(a, x, y):
    return manual(y,x,a)
def manual(y,x,a):
    __list=list()
    for __tuple in zip(y,x):
        __list.append(__tuple[0]*a+__tuple[1])
    return __list
"""
        self.run_test(
            code,
            1.5, [1,2,3], [0.,2.,4.],
            saxpy4=[float,List[int], List[float]])

    def test_sxpy(self):
        code="""
def sxpy(x, y):
    def duad(xi, yi): return xi + yi
    return list(map(duad, x, y))
"""
        self.run_test(
            code,
            [1,2,3], [0.,2.,4.],
            sxpy=[List[int], List[float]])

    def test_incr(self):
        self.run_test(
            "def incr(x): return list(map(lambda xi: xi + 1, x))",
            [0., 0., 0.],
            incr=[List[float]])

    def test_as_ones(self):
        self.run_test(
            "def as_ones(x): return list(map(lambda xi: 1, x))",
            [0., 0., 0.],
            as_ones=[List[float]])

    def test_idm(self):
        self.run_test(
            "def idm(x): return list(map(lambda b: b, x))",
            [1, 2, 3],
            idm=[List[int]])

    def test_incr_list(self):
        self.run_test(
            "def incr_list(x): return [xi + 1 for xi in x]",
            [1., 2., 3.],
            incr_list=[List[float]])


    def test_idx(self):
        code="""
def idx(x):
    def id(xi): return xi
    return list(map(id, x))"""
        self.run_test(code, [1,2,3], idx=[List[int]])

    def test_rbf(self):
        code="""
from math import exp
def norm2_diff(x, y):
   def el(xi, yi):
       diff = xi - yi
       return diff * diff
   return sum(map(el, x, y))

def rbf(ngamma, x, y):
   return exp(ngamma * norm2_diff(x,y))"""
        self.run_test(
            code,
            2.3, [1,2,3], [1.1,1.2,1.3],
            rbf=[float, List[int], List[float]])

# from copperhead-new/copperhead/prelude.py
    def test_indices(self):
        self.run_test(
            "def indices(A):return list(range(len(A)))",
            [1,2],
            indices=[List[int]])

    def test_gather(self):
        self.run_test(
            "def gather(x, indices): return [x[i] for i in indices]",
            [1,2,3,4,5], [0,2,4],
            gather=[List[int], List[int]])

    def test_scatter(self):
        code="""
def indices(x): return list(range(len(x)))
def scatter(src, indices_, dst):
    assert len(src)==len(indices_)
    result = list(dst)
    for i in range(len(src)):
        result[indices_[i]] = src[i]
    return result
"""
        self.run_test(
            code,
            [0.0,1.0,2.,3.,4.,5.,6.,7.,8.,9.],[5,6,7,8,9,0,1,2,3,4],[0,0,0,0,0,0,0,0,0,0,18],
            scatter=[List[float], List[int], List[int]])

    def test_scan(self):
        code="""
def prefix(A): return scan(lambda x,y:x+y, A)
def scan(f, A):
    B = list(A)
    for i in range(1, len(B)):
        B[i] = f(B[i-1], B[i])
    return B
"""
        self.run_test(code, [1.,2.,3.], prefix=[List[float]])



# from Copperhead: Compiling an Embedded Data Parallel Language
# by Bryan Catanzaro, Michael Garland and Kurt Keutzer
# http://www.eecs.berkeley.edu/Pubs/TechRpts/2010/EECS-2010-124.html

    def test_spvv_csr(self):
        code="""
def spvv_csr(x, cols, y):
    def gather(x, indices): return [x[i] for i in indices]
    z = gather(y, cols)
    return sum(map(lambda a, b: a * b, x, z))
"""
        self.run_test(code, [1,2,3],[0,1,2],[5.5,6.6,7.7], spvv_csr=[List[int], List[int], List[float]])

    def test_spmv_csr(self):
        code="""
def spvv_csr(x, cols, y):
    def gather(x, indices): return [x[i] for i in indices]
    z = gather(y, cols)
    return sum(map(lambda a, b: a * b, x, z))
def spmv_csr(Ax, Aj, x):
    return list(map(lambda y, cols: spvv_csr(y, cols, x), Ax, Aj))
"""
        self.run_test(code, [[0,1,2],[0,1,2],[0,1,2]],[[0,1,2],[0,1,2],[0,1,2]],[0,1,2], spmv_csr=[List[List[int]], List[List[int]], List[int]])

    def test_spmv_ell(self):
        code="""
def indices(x): return range(len(x))
def spmv_ell(data, idx, x):
    def kernel(i):
        return sum(map(lambda Aj, J: Aj[i] * x[J[i]], data, idx))
    return list(map(kernel, indices(x)))
"""
        self.run_test(code, [[0,1,2],[0,1,2],[0,1,2]],[[0,1,2],[0,1,2],[0,1,2]],[0,1,2], spmv_ell=[List[List[int]], List[List[int]], List[int]])

    def test_vadd(self):
        self.run_test("def vadd(x, y): return list(map(lambda a, b: a + b, x, y))", [0.,1.,2.],[5.,6.,7.], vadd=[List[float], List[float]])

    def test_vmul(self):
        self.run_test("def vmul(x, y): return list(map(lambda a, b: a * b, x, y))", [0.,1.,2.],[5.,6.,7.], vmul=[List[float], List[float]])

    def test_form_preconditioner(self):
        code="""
def vadd(x, y): return list(map(lambda a, b: a + b, x, y))
def vmul(x, y): return list(map(lambda a, b: a * b, x, y))
def form_preconditioner(a, b, c):
    def det_inverse(ai, bi, ci):
        return 1.0/(ai * ci - bi * bi)
    indets = list(map(det_inverse, a, b, c))
    p_a = vmul(indets, c)
    p_b = list(map(lambda a, b: -a * b, indets, b))
    p_c = vmul(indets, a)
    return p_a, p_b, p_c
"""
        self.run_test(code, [1,2,3],[0,1,2],[5.5,6.6,7.7],form_preconditioner=[List[int], List[int], List[float]])

    def test_precondition(self):
        code="""
def precondition(u, v, p_a, p_b, p_c):
    def vadd(x, y): return map(lambda a, b: a + b, x, y)
    def vmul(x, y): return map(lambda a, b: a * b, x, y)
    e = vadd(vmul(p_a, u), vmul(p_b, v))
    f = vadd(vmul(p_b, u), vmul(p_c, v))
    return list(e), list(f)
"""
        self.run_test(code, [1,2,3], [5.5,6.6,7.7],[1,2,3], [5.5,6.6,7.7],[8.8,9.9,10.10], precondition=[List[int], List[float], List[int], List[float], List[float]])




import omp

def omp_single_private():
    nr_threads_in_single = 0
    nr_iterations = 0
    result = 0
    LOOPCOUNT = 1000

    if 'omp parallel':
        use_parallel = omp.in_parallel()
        myresult = 0
        myit = 0
        for i in range(LOOPCOUNT):
            if 'omp single private(nr_threads_in_single)':
                nr_threads_in_single = 0
                'omp flush'
                nr_threads_in_single += 1
                'omp flush'
                myit += 1
                myresult += nr_threads_in_single
        if 'omp critical':
            result += nr_threads_in_single
            nr_iterations += myit

    return not use_parallel or (result == 0 and nr_iterations == LOOPCOUNT)



#from http://rosettacode.org/wiki/Permutation_test#Python
#pythran export permutationTest(int list, int list)
#pythran export permutationTest2(int list, int list)
#runas permutationTest([85, 88, 75, 66, 25, 29, 83, 39, 97], [68, 41, 10, 49, 16, 65, 32, 92, 28, 98])
#runas permutationTest2([85, 88, 75, 66, 25, 29, 83, 39, 97], [68, 41, 10, 49, 16, 65, 32, 92, 28, 98])
from itertools import combinations as comb

def statistic(ab, a):
    sumab, suma = sum(ab), sum(a)
    return ( suma / len(a) - (sumab -suma) / (len(ab) - len(a)) )

def permutationTest(a, b):
    ab = a + b
    Tobs = statistic(ab, a)
    under = 0
    for count, perm in enumerate(comb(ab, len(a)), 1):
        if statistic(ab, perm) <= Tobs:
            under += 1
    return under * 100. / count

def permutationTest2(a, b):
    ab = a + b
    Tobs = sum(a)
    under = 0
    for count, perm in enumerate(comb(ab, len(a)), 1):
        if sum(perm) <= Tobs:
            under += 1
    return under * 100. / count



#runas solve(1000)
# pythran export solve(int)
def solve(n):
    """
    If p is the perimeter of a right angle triangle with integral length sides,
    {a,b,c}, there are exactly three solutions for p = 120.

    {20,48,52}, {24,45,51}, {30,40,50}

    For which value of p < 1000, is the number of solutions maximised?
    """

    maxp, maxsol = 0, 0
    for p in range(12, n + 1, 2):
        solutions = 0
        # a < b < c. So a is at most 1/3 of p. b is between a and (p-a)/2
        for a in range(1, p//3):
            a2 = a*a
            for b in range(a, (p-a)//2):
                c = p - a - b
                if a2 + b*b == c*c:
                    solutions = solutions + 1
        if solutions > maxsol:
            maxp, maxsol = p, solutions

    return maxp



from pythran.tests import TestEnv


@TestEnv.module
class TestOs(TestEnv):

    def test_os_path(self):
        self.run_test('def os_path(l): import os; return os.path.join(l)',
                      "mypath", os_path=[str])

    def test_os_path2(self):
        self.run_test('def os_path2(l): import os; return os.path.join("lili", l)',
                      "mypath", os_path2=[str])

    def test_os_path3(self):
        self.run_test('def os_path3(l): import os; return os.path.join("lili", l)',
                      "/mypath", os_path3=[str])

    def test_os_path4(self):
        self.run_test('def os_path4(l): import os; return os.path.join("lili/", l)',
                      "mypath", os_path4=[str])

    def test_os_path5(self):
        self.run_test('def os_path5(l): import os; return os.path.join("", l)',
                      "mypath", os_path5=[str])



#from http://rosettacode.org/wiki/Ackermann_function#Python
#pythran export ack1(int, int)
#pythran export ack2(int, int)
#pythran export ack3(int, int)
#runas ack1(2, 2)
#runas ack2(2, 1)
#runas ack3(1, 2)

def ack1(M, N):
   return (N + 1) if M == 0 else (
         ack1(M-1, 1) if N == 0 else ack1(M-1, ack1(M, N-1)))


def ack2(M, N):
    if M == 0:
        return N + 1
    elif N == 0:
        return ack1(M - 1, 1)
    else:
        return ack1(M - 1, ack1(M, N - 1))


def ack3(M, N):
   return (N + 1)   if M == 0 else (
          (N + 2)   if M == 1 else (
          (2*N + 3) if M == 2 else (
          (8*(int(2**N) - 1) + 5) if M == 3 else ( # int conversion is pythran-specific
          ack2(M-1, 1) if N == 0 else ack2(M-1, ack2(M, N-1))))))



import unittest
from pythran.tests import TestFromDir
import os
import glob

class TestEuler(TestFromDir):
    path = os.path.join(os.path.dirname(__file__),"euler")
    files = glob.glob(os.path.join(path,"euler*.py"))

TestEuler.populate(TestEuler)

if __name__ == '__main__':
    unittest.main()



#runas solve()
# pythran export solve()
def solve():
    """
    145 is a curious number, as 1! + 4! + 5! = 1 + 24 + 120 = 145.

    Find the sum of all numbers which are equal to the sum of the factorial of
    their digits.

    Note: as 1! = 1 and 2! = 2 are not sums they are not included.
    """
    fact = [1, 1, 2, 6, 24, 120, 720, 5040, 40320, 362880]

    def sum_of_digits_factorial(n):
        s = 0
        while n > 0:
            d = n % 10
            s = s + fact[d]
            n = n // 10
        return s

    return sum(n for n in range(10, 100000) if n == sum_of_digits_factorial(n))



#pythran export matrix_class_distance(float64[:,:], int[], float64[:,:], int)
#from https://stackoverflow.com/questions/59601987
#runas import numpy as np; n = 200;d = 10;iterations = 20;np.random.seed(42);dat = np.random.random((n, d));dat_filter = np.random.randint(0, n, size=n); dat_points = np.random.random((n, d)); matrix_class_distance(dat, dat_filter, dat_points, iterations)
import numpy as np
def matrix_class_distance(dat, dat_filter, dat_points, iterations):
    aggregation = 0
    for i in range(iterations):
        aggregation += np.sum(np.linalg.norm(dat[dat_filter==i] - dat_points[i], axis=1))
    return aggregation



# taken from http://oddbloke.uwcs.co.uk/parallel_benchmarks/
#pythran export sum_primes(int)
#runas sum_primes(200)
#bench sum_primes(320000)
import math
def isprime(n):
    """Returns True if n is prime and False otherwise"""
    if n < 2:
        return False
    if n == 2:
        return True
    max = int(math.ceil(math.sqrt(n)))
    i = 2
    while i <= max:
        if n % i == 0:
            return False
        i += 1
    return True

def sum_primes(n):
    """Calculates sum of all primes below given integer n"""
    return sum([x for x in range(2,n) if isprime(x)])



#from http://rosettacode.org/wiki/Palindrome_detection#Python
#pythran export test()
#runas test()

def is_palindrome(s):
    return s == s[::-1]

def is_palindrome_r2(s):
    return not s or s[0] == s[-1] and is_palindrome_r2(s[1:-1])

def is_palindrome_r(s):
    if len(s) <= 1:
        return True
    elif s[0] != s[-1]:
        return False
    else:
        return is_palindrome_r(s[1:-1])

def test_(f, good, bad):
    if all(f(x) for x in good) and not any(f(x) for x in bad):
        print('function passed all %d tests' % (len(good)+len(bad)))

def test():
    pals = ['', 'a', 'aa', 'aba', 'abba']
    notpals = ['aA', 'abA', 'abxBa', 'abxxBa']
    test_(is_palindrome, pals, notpals)
    test_(is_palindrome_r, pals, notpals)
    test_(is_palindrome_r2, pals, notpals)



#pythran export quicksort(int list, int, int)
#runas quicksort(list(range(10)),0,9)
def partition(list, start, end):
    pivot = list[end]                          # Partition around the last value
    bottom = start-1                           # Start outside the area to be partitioned
    top = end                                  # Ditto

    done = 0
    while not done:                            # Until all elements are partitioned...

        while not done:                        # Until we find an out of place element...
            bottom = bottom+1                  # ... move the bottom up.

            if bottom == top:                  # If we hit the top...
                done = 1                       # ... we are done.
                break

            if list[bottom] > pivot:           # Is the bottom out of place?
                list[top] = list[bottom]       # Then put it at the top...
                break                          # ... and start searching from the top.

        while not done:                        # Until we find an out of place element...
            top = top-1                        # ... move the top down.
            if top == bottom:                  # If we hit the bottom...
                done = 1                       # ... we are done.
                break

            if list[top] < pivot:              # Is the top out of place?
                list[bottom] = list[top]       # Then put it at the bottom...
                break                          # ...and start searching from the bottom.

    list[top] = pivot                          # Put the pivot in its place.
    return top                                 # Return the split point


def do_quicksort(list, start, end):
    if start < end:                            # If there are two or more elements...
        split = partition(list, start, end)    # ... partition the sublist...
        do_quicksort(list, start, split-1)        # ... and sort both halves.
        do_quicksort(list, split+1, end)

def quicksort(l,s,e):
    do_quicksort(l,s,e)



def omp_single_copyprivate():
    result = 0
    nr_iterations = 0
    LOOPCOUNT = 1000
    j = 0

    if 'omp parallel private(j)':
        for i in range(LOOPCOUNT):
            if 'omp single copyprivate(j)':
                nr_iterations += 1
                j = i
            if 'omp critical':
                result += j - i
            'omp barrier'
    return result == 0 and nr_iterations == LOOPCOUNT




def omp_atomic_bitor():
    sum = 0
    LOOPCOUNT = 1000
    logics = [1]*LOOPCOUNT
    logics[LOOPCOUNT//2] = 0
    "omp parallel for"
    for i in range(LOOPCOUNT):
        "omp atomic"
        sum |= logics[i]
    return sum == 1



#from http://rosettacode.org/wiki/Permutations/Derangements#Python
from itertools import permutations
import math

#pythran export test(int, int, int)
#runas test(4, 10, 20)

def derangements(n):
    'All deranged permutations of the integers 0..n-1 inclusive'
    return ( perm for perm in permutations(range(n))
             if all(indx != p for indx, p in enumerate(perm)) )

def subfact(n):
    if n == 2 or n == 0:
        return 1
    elif n == 1:
        return 0
    elif  1 <= n <=18:
        return round(math.factorial(n) / math.e)
    elif n.imag == 0 and n.real == int(n.real) and n > 0:
        return (n-1) * ( subfact(n - 1) + subfact(n - 2) )
    else:
        raise ValueError()

def _iterlen(iter):
    'length of an iterator without taking much memory'
    l = 0
    for _ in iter:
        l += 1
    return l

def test(n1, n2, n3):
    print("Derangements of %s" % list(range(n1)))
    for d in derangements(n1):
        print("  %s" % (d,))
    print("\nTable of n vs counted vs calculated derangements")
    for n in range(n2):
        print("%2i %-5i %-5i" %
              (n, _iterlen(derangements(n)), subfact(n)))
    print("\n!%i = %i" % (n3, subfact(n3)))



"""
`reconstruction_loop` originally part of CellProfiler,
code licensed under both GPL and BSD licenses.

Website: http://www.cellprofiler.org
Copyright (c) 2003-2009 Massachusetts Institute of Technology
Copyright (c) 2009-2011 Broad Institute
All rights reserved.
Original author: Lee Kamentsky

"""

# pythran export reconstruction_loop(uint32[], int32[], int32[], int32[],
#                                    intp, intp)


def reconstruction_loop(ranks, prev, next, strides, current_idx, image_stride):
    """The inner loop for reconstruction.

    This algorithm uses the rank-order of pixels. If low intensity pixels have
    a low rank and high intensity pixels have a high rank, then this loop
    performs reconstruction by dilation. If this ranking is reversed, the
    result is reconstruction by erosion.

    For each pixel in the seed image, check its neighbors. If its neighbor's
    rank is below that of the current pixel, replace the neighbor's rank with
    the rank of the current pixel. This dilation is limited by the mask, i.e.
    the rank at each pixel cannot exceed the mask as that pixel.

    Parameters
    ----------
    ranks : array
        The rank order of the flattened seed and mask images.
    prev, next: arrays
        Indices of previous and next pixels in rank sorted order.
    strides : array
        Strides to neighbors of the current pixel.
    current_idx : int
        Index of highest-ranked pixel used as starting point in loop.
    image_stride : int
        Stride between seed image and mask image in `aranks`.
    """
    nstrides = strides.shape[0]

    while current_idx != -1:
        if current_idx < image_stride:
            current_rank = ranks[current_idx]
            if current_rank == 0:
                break
            for i in range(nstrides):
                neighbor_idx = current_idx + strides[i]
                neighbor_rank = ranks[neighbor_idx]
                # Only propagate neighbors ranked below the current rank
                if neighbor_rank < current_rank:
                    mask_rank = ranks[neighbor_idx + image_stride]
                    # Only propagate neighbors ranked below the mask rank
                    if neighbor_rank < mask_rank:
                        # Raise the neighbor to the mask rank if
                        # the mask ranked below the current rank
                        if mask_rank < current_rank:
                            current_link = neighbor_idx + image_stride
                            ranks[neighbor_idx] = mask_rank
                        else:
                            current_link = current_idx
                            ranks[neighbor_idx] = current_rank
                        # unlink the neighbor
                        nprev = prev[neighbor_idx]
                        nnext = next[neighbor_idx]
                        next[nprev] = nnext
                        if nnext != -1:
                            prev[nnext] = nprev
                        # link to the neighbor after the current link
                        nnext = next[current_link]
                        next[neighbor_idx] = nnext
                        prev[neighbor_idx] = current_link
                        if nnext >= 0:
                            prev[nnext] = neighbor_idx
                            next[current_link] = neighbor_idx
        current_idx = next[current_idx]



#from https://groups.google.com/forum/#!topic/parakeet-python/p-flp2kdE4U
#pythran export slowparts(int, int, float [][][], float [][][], float [][], float [][], float [][][], float [][][], int)
#runas import numpy as np ;d = 10 ;re = 5 ;params = (d, re, np.ones((2*d, d+1, re)), np.ones((d, d+1, re)),  np.ones((d, 2*d)), np.ones((d, 2*d)), np.ones((d+1, re, d)), np.ones((d+1, re, d)), 1) ; slowparts(*params)
#bench import numpy as np ;d = 87 ;re = 5 ;params = (d, re, np.ones((2*d, d+1, re)), np.ones((d, d+1, re)),  np.ones((d, 2*d)), np.ones((d, 2*d)), np.ones((d+1, re, d)), np.ones((d+1, re, d)), 1) ; slowparts(*params)
from numpy import zeros, power, tanh
def slowparts(d, re, preDz, preWz, SRW, RSW, yxV, xyU, resid):
    """ computes the linear algebra intensive part of the gradients of the grae
    """
    fprime = lambda x: 1 - power(tanh(x), 2)

    partialDU = zeros((d+1, re, 2*d, d))
    for k in range(2*d):
        for i in range(d):
            partialDU[:,:,k,i] = fprime(preDz[k]) * fprime(preWz[i]) * (SRW[i,k] + RSW[i,k]) * yxV[:,:,i]

    return partialDU



import numpy as np

#pythran export _sum_abs_axis0(float[:,:])
def _sum_abs_axis0(X):
    block_size = 2**20
    r = None
    for j in range(0, X.shape[0], block_size):
        y = np.sum(np.abs(X[j:j+block_size]), axis=0)
        if r is None:
            r = y
        else:
            r += y
    return r



#from http://rosettacode.org/wiki/Perfect_numbers#Python
#pythran export perf(int)
#pythran export perf_(int)
#runas list(map(perf, range(20)))
#runas list(map(perf_, range(20)))

def perf(n):
    sum = 0
    for i in range(1, n):
        if n % i == 0:
            sum += i
    return sum == n

def perf_(n):
    return n == sum(i for i in range(1, n) if n % i == 0)



def omp_critical():
    sum = 0
    if 'omp parallel':
        mysum = 0
        'omp for'
        for i in range(1000):
            mysum += i
        'omp critical'
        sum += mysum
    return sum == 999 * 1000 / 2



#pythran export convnet(float32[:,:], float32[:,:], int, int, int)
#runas import numpy as np; x = np.arange(16., dtype=np.float32).reshape(4,4); y= np.arange(16., dtype=np.float32).reshape(4,4); convnet(x, y, 4,1,1)
import numpy as np
def sigmoid(z):
    return 1 / (1 + np.exp(-z))


def convnet(conv_matrix, qcnn_filter, length, batch_size, state_size):
    state_size_2 = state_size * 2
    state_size_3 = state_size * 3
    state_size_4 = state_size * 4

    conv_results = np.dot(conv_matrix, qcnn_filter).reshape(length, batch_size, state_size_4)
    conv_results[:, :, :state_size] = np.tanh(conv_results[:, :, :state_size])
    conv_results[:, :, state_size:state_size_4] = sigmoid(conv_results[:, :, state_size:state_size_4])

    state_results = np.zeros((batch_size, length, state_size_2), dtype=np.float32)
    state = np.zeros((batch_size, state_size), dtype=np.float32)
    for i in range(length):
        z = conv_results[i, :, :state_size]
        f = conv_results[i, :, state_size:state_size_2]
        o = conv_results[i, :, state_size_2:state_size_3]
        state = f * state + (1 - f) * z
        state_results[:, i, :state_size] = state * o

    state = np.zeros((batch_size, state_size), dtype=np.float32)
    for i in range(length - 1, -1, -1):
        z = conv_results[i, :, :state_size]
        f = conv_results[i, :, state_size_3:state_size_4]
        o = conv_results[i, :, state_size_2:state_size_3]
        state = f * state + (1 - f) * z
        state_results[:, i, state_size:] = state * o
    state_results = state_results.reshape((batch_size * length, state_size_2))
    return state_results



def omp_parallel_sections_firstprivate():
    sum = 7
    sum0 = 11

    if 'omp parallel sections firstprivate(sum0)':
        if 'omp section':
            if 'omp critical':
                sum += sum0
        if 'omp section':
            if 'omp critical':
                sum += sum0
        if 'omp section':
            if 'omp critical':
                sum += sum0
    known_sum=11*3+7
    return (known_sum==sum)



def omp_master():
    import omp
    threads = 0
    executing_thread = -1

    if 'omp parallel':
        if 'omp master':
            if 'omp critical':
                threads += 1
            executing_thread = omp.get_thread_num()
    return threads == 1 and executing_thread == 0



#runas solve()
#pythran export solve()
'''
 Using names.txt (right click and 'Save Link/Target As...'), a 46K text file containing over five-thousand first names, begin by sorting it into alphabetical order. Then working out the alphabetical value for each name, multiply this value by its alphabetical position in the list to obtain a name score.

 For example, when the list is sorted into alphabetical order, COLIN, which is worth 3 + 15 + 12 + 9 + 14 = 53, is the 938th name in the list. So, COLIN would obtain a score of 938 x 53 = 49714.

 What is the total of all the name scores in the file?
 '''


def worth(name):
	return sum(ord(letter) - ord('A') + 1 for letter in name)
def solve():
 names = open('pythran/tests/euler/names22.txt').read().replace('"', '').split(',')
 names.sort()

 return sum((i+1) * worth(names[i]) for i in range(0, len(names)))



#from http://stackoverflow.com/questions/11641098/interpreting-a-benchmark-in-c-clojure-python-ruby-scala-and-others
#pythran export primes_below(int)
#runas primes_below(1000)
#bench primes_below(15000)
def is_prime(n):
      return all((n%j > 0) for j in range(2, n))

def primes_below(x):
        return [[j-6, j] for j in range(9, x+1) if is_prime(j) and is_prime(j-6)]




#encoding: utf8
from pythran.tests import TestEnv

from unittest import skip, skipIf
import numpy

from pythran.typing import *

class TestAdvanced(TestEnv):

    def test_generator_enumeration(self):
        code = '''
            def dummy_generator(l):
                for i in l: yield i
            def generator_enumeration(begin, end):
                return [i for i in enumerate(dummy_generator(range(begin,end)))]'''
        self.run_test(code, 2, 10, generator_enumeration=[int, int])

    def test_augassign_floordiv(self):
        self.run_test("def augassign_floordiv(i,j): k=i ; k//=j; return k",
                2, 5, augassign_floordiv=[int, int])

    def test_builtin_constructors(self):
        self.run_test("def builtin_constructors(l): return list(map(int,l))",
                [1.5, 2.5], builtin_constructors=[List[float]])

    def test_tuple_sum(self):
        self.run_test("def tuple_sum(tpl): return sum(tpl)", (1, 2, 3.5), tuple_sum=[Tuple[int, int, float]])

    def test_minus_unary_minus(self):
        self.run_test("def minus_unary_minus(a): return a - -1", 1, minus_unary_minus=[int])

    def test_bool_op_casting(self):
        self.run_test('''
            def bool_op_casting():
                l=[]
                L=[1]
                M=[2]
                if (l and L) or M:
                    return (l and L) or M
                else:
                    return M''', bool_op_casting=[])

    def test_map_on_generator(self):
        self.run_test('def map_on_generator(l): return list(map(float,(x*x for x in l)))', [1,2,3], map_on_generator=[List[int]])

    def test_map2_on_generator(self):
        self.run_test('def map2_on_generator(l): return list(map(lambda x,y : x*y, l, (y for x in l for y in l if x < 1)))', [0,1,2,3], map2_on_generator=[List[int]])

    def test_enumerate_on_generator(self):
        self.run_test("def enumerate_on_generator(n): return list(map(lambda z: z[0], enumerate((y for x in range(n) for y in range(x)))))", 5, enumerate_on_generator=[int])

    def test_enumerate_iterate(self):
        self.run_test("""
                      def enumerate_iterate(n):
                        s = 0
                        for x in enumerate(n):
                          for y in x:
                            s += y
                        return s""",
                      [5, 6],
                      enumerate_iterate=[List[int]])

    def test_max_interface_arity(self):
        self.run_test('def max_interface_arity({0}):pass'.format(', '.join('_'+str(i) for i in range(42))), *list(range(42)), max_interface_arity=[int]*42)

    def test_max_kwonly_key(self):
        self.run_test('def max_kwonly_key(x): return max(x, key=lambda x:-x)',
                      list(range(42)), max_kwonly_key=[List[int]])

    def test_multiple_max(self):
        self.run_test('def multiple_max(i,j,k): return max(i,j,k)', 1, 1.5, False, multiple_max=[int, float, bool])

    def test_zip_on_generator(self):
        self.run_test('def zip_on_generator(n): return list(zip((i for i in range(n)), (i*2 for i in range(1,n+1))))', 5, zip_on_generator=[int])

    def test_parallel_enumerate(self):
        self.run_test('def parallel_enumerate(l):\n k = [0]*(len(l) + 1)\n "omp parallel for"\n for i,j in enumerate(l):\n  k[i+1] = j\n return k', list(range(1000)), parallel_enumerate=[List[int]])

    def test_ultra_nested_functions(self):
        code = '''
def ultra_nested_function(n):
	def foo(y):
		def bar(t): return t
		return bar(y)
	return foo(n)'''
        self.run_test(code, 42, ultra_nested_function=[int])

    def test_generator_sum(self):
        code = '''
def generator_sum(l0,l1):
    return sum(x*y for x,y in zip(l0,l1))'''
        self.run_test(code, list(range(10)), list(range(10)), generator_sum=[List[int],List[int]])

    def test_tuple_to_list(self):
        self.run_test('def tuple_to_list(t): return list(t)', (1,2,3), tuple_to_list=[Tuple[int, int, int]])

    def test_in_generator(self):
        self.run_test("def in_generator(n):return 1. in (i*i for i in range(n))", 5, in_generator=[int])

    def test_tuple_unpacking_in_generator(self):
        code = '''
def foo(l):
    a, b = 1,0
    yield a
    yield b
def tuple_unpacking_in_generator(n):
    f = foo(range(n))
    return 0 in f'''
        self.run_test(code, 10, tuple_unpacking_in_generator=[int])

    def test_loop_tuple_unpacking_in_generator(self):
        code= '''
def foo(l):
    for i,j in enumerate(l):
        yield i,j
def loop_tuple_unpacking_in_generator(n):
    f = foo(range(n))
    return (0,0) in f'''
        self.run_test(code, 10, loop_tuple_unpacking_in_generator=[int])

    def test_assign_in_except(self):
        code = '''
def assign_in_except():
    try:
        a=1
    except:
        a+=a
    return a'''
        self.run_test(code, assign_in_except=[])

    def test_combiner_on_empty_list(self):
        code = '''
def b(l):
    l+=[1]
    return l
def combiner_on_empty_list():
    return b(list()) + b([])'''
        self.run_test(code, combiner_on_empty_list=[])

    def test_dict_comprehension_with_tuple(self):
        self.run_test('def dict_comprehension_with_tuple(n): return { x:y for x,y in zip(range(n), range(1+n)) }', 10, dict_comprehension_with_tuple=[int])

    def test_nested_comprehension_with_tuple(self):
        self.run_test('def nested_comprehension_with_tuple(l): return [[ x+y for x,y in sqrpoints ] for sqrpoints in l]', [[(x,x)]*5 for x in list(range(10))], nested_comprehension_with_tuple=[List[List[Tuple[int,int]]]])

    def test_hashable_tuple(self):
        self.run_test('def hashable_tuple(): return { (1,"e", 2.5) : "r" }', hashable_tuple=[])

    def test_conflicting_names(self):
        self.run_test('def map(): return 5', map=[])

    def test_multiple_compares(self):
        self.run_test('def multiple_compares(x): return 1 < x < 2, 1 < x + 1 < 2', 0.5, multiple_compares=[float])

    def test_default_arg0(self):
        self.run_test('def default_arg0(n=12): return n', default_arg0=[])

    def test_default_arg1(self):
        self.run_test('def default_arg1(m,n=12): return m+n', 1, default_arg1=[int])

    def test_default_arg2(self):
        self.run_test('def default_arg2(n=12): return n', 1, default_arg2=[int])

    def test_default_arg3(self):
        self.run_test('def default_arg3(m,n=12): return m+n', 1, 2, default_arg3=[int,int])

    def test_default_arg4(self):
        code = '''
            import numpy as np
            def default_arg4(signal,sR):
                N = 30
                F = 0.
                F2 = 22000
                FF = 10
                W = test2(sR, FF, N, F, F2)
                return W

            def test2(sr,N,M=128,F=0.0,F2=0,B=False,No=1):
                W = np.zeros(10)
                return W'''
        self.run_test(code, 1, 2, default_arg4=[int,int])

    def test_default_arg5(self):
        self.run_test('import numpy\ndef default_arg5(m,n=-numpy.inf): return m, n', 1, default_arg5=[int])

    def test_default_arg6(self):
        code = 'from numpy import empty\ndef default_arg6(x=empty(3)):pass'
        with self.assertRaises(SyntaxError):
            self.run_test(code, default_arg6=[])

    @skip("lists as zeros parameter are not supported")
    def test_list_as_zeros_parameter(self):
        self.run_test('def list_as_zeros_parameter(n): from numpy import zeros ; return zeros([n,n])', 3, list_as_zeros_parameter=[int])

    def test_add_arrays(self):
        self.run_test('def add_arrays(s): return (s,s) + (s,)', 1, add_arrays=[int])

    def test_tuple_to_tuple(self):
        self.run_test('def tuple_to_tuple(t): return tuple((1, t))',
                      '2',
                      tuple_to_tuple=[str])

    def test_array_to_tuple(self):
        self.run_test('def array_to_tuple(t): return tuple((1, t))',
                      2,
                      array_to_tuple=[int])

    def test_list_to_tuple(self):
        self.run_test('def list_to_tuple(t): return tuple([1, t])',
                      2,
                      list_to_tuple=[int])

    def test_tuple_to_shape(self):
        self.run_test('def tuple_to_shape(n): from numpy import zeros; return zeros((n,4))', 5, tuple_to_shape=[int])

    def test_print_intrinsic(self):
        self.run_test('def print_intrinsic(): print(len)',
                      print_intrinsic=[])

    def test_function_redefinition(self):
        code = 'def function_redefinition(x):pass\ndef function_redefinition():pass'
        with self.assertRaises(SyntaxError):
            self.run_test(code, function_redefinition=[])

    def test_global_redefinition(self):
        code = 'foo=0\nfoo=1\ndef global_redefinition(x):pass'
        with self.assertRaises(SyntaxError):
            self.run_test(code, global_redefinition=[])

    def test_global_update(self):
        code = 'foo=[]\ndef global_update(x): x.append(1)'
        with self.assertRaises(SyntaxError):
            self.run_test(code, global_update=[])

    def test_invalid_call0(self):
        code = 'def foo(x):pass\ndef invalid_call0(): return foo()'
        with self.assertRaises(SyntaxError):
            self.run_test(code, invalid_call0=[])

    def test_invalid_call1(self):
        code = 'def foo(x=1):pass\ndef invalid_call1(l): return foo(l,l)'
        with self.assertRaises(SyntaxError):
            self.run_test(code, 1, invalid_call1=[int])

    def test_invalid_call2(self):
        code = 'def foo(x):pass\ndef bar():pass\ndef invalid_call2(l): return (foo if l else bar)(l)'
        with self.assertRaises(SyntaxError):
            self.run_test(code, 1, invalid_call2=[int])

    def test_ellipsis(self):
        code = 'def ellipsis_(x): return x[...,1]'
        with self.assertRaises(SyntaxError):
            self.run_test(code, numpy.ones((3,3)), ellipsis=[NDArray[float,:,:]])

    def test_multiple_lambda(self):
        code = '''
            def multiple_lambda(x):
                g = lambda : x
                return foo(g)
            def foo(t):
                g = lambda : 1
                return t() + g()
            '''
        self.run_test(code, 1, multiple_lambda=[int])

    def test_function_with_non_ascii_docstring(self):
        code = '''
            def function_with_non_ascii_docstring():
                'éàea'
        '''
        self.run_test(code, function_with_non_ascii_docstring=[])

    def test_matmul_operator(self):
        code = 'def matmul_operator(x, y): return x @ y'
        self.run_test(
            code,
            numpy.array([[1., 1.], [2., 2.]]),
            numpy.array([[0., 2.], [1., 3.]]),
            matmul_operator=[NDArray[float, :,:], NDArray[float, :,:]])

    def test_generator_handler_name(self):
        code = '''
            def foo(x):
                for i in range(x):
                    if i > 1:
                        break
                    yield i

            def generator_handler_name(n):
                return list(foo(n))'''
        self.run_test(code,
                      3,
                      generator_handler_name=[int])

    def test_generator_handler_name2(self):
        code = '''
            def foo(x):
                for i in ["1"] * x:
                    if len(i) == 1:
                        break
                    yield i

            def generator_handler_name2(n):
                return list(foo(n))'''
        self.run_test(code,
                      3,
                      generator_handler_name2=[int])

    def test_builtin_slices(self):
        code = '''
            def builtin_slices(x):
                s = slice(2, None, None)
                return (s.start, s.stop, s.step, s,
                        x[s],
                        x[slice(3)],
                        x[slice(1,2)],
                        x[slice(1,10,2)],
                        x[slice(3, None)],
                        x[slice(None,4)],
                        x[slice(None,4, None)])'''
        self.run_test(code,
                      numpy.arange(15),
                      builtin_slices=[NDArray[int,:]])

    def test_slicing_tuple(self):
        code = '''
            def testFunc():
                x=2
                y=3
                z=4
                return x,y,z

            def slicing_tuple(n):
                x,y = testFunc()[0:n]
                return x,y'''
        self.run_test(code, 2, slicing_tuple=[int])

    def test_static_list0(self):
        code = '''
            def static_list0(n):
                s = list(n)
                s[1] = 1
                return tuple(s)'''
        self.run_test(code, (2, 2), static_list0=[Tuple[int, int]])

    def test_static_list1(self):
        code = '''
            def foo(x, y):
                return len(y) + x
            def static_list1(n):
                s = list(n)
                s[1] = foo(len(s), s)
                return tuple(s)'''
        self.run_test(code, (2, 2), static_list1=[Tuple[int, int]])

    def test_static_list2(self):
        code = '''
            def static_list2(t0, t1):
                s = [slice(x, y) for x,y in zip(t0, t1)]
                return tuple(s)'''
        self.run_test(code, (2, 2), (3,3), static_list2=[Tuple[int, int], Tuple[int, int]])

    def test_static_list3(self):
        code = '''
            import numpy as np

            def StridedSlice(x,begins, ends, strides):
                slices = tuple([slice(b, e if e else None, s) for b, e, s in zip(begins,ends, strides)])
                return x[slices]

            def static_list3(x):
                return StridedSlice(x,[0,2,3], [5,0,7], [1,1,1])'''
        self.run_test(code, numpy.arange(1000).reshape(10,10,10), static_list3=[NDArray[int, :,:,:]])

    def test_static_list4(self):
        code = '''
            import numpy as np

            def StridedSlice(x,begins, ends, strides):
                slices = tuple([slice(b, e if e else None, s) for b, e, s in zip(begins,ends, strides)])
                return x[slices]

            def static_list4(x):
                return StridedSlice(x,np.array([0,2,3]), np.array([5,0,7]), [1,1,1])'''
        self.run_test(code, numpy.arange(1000).reshape(10,10,10), static_list4=[NDArray[int, :,:,:]])

    def test_tuple_slicing0(self):
        code = '''
            def tuple_slicing0(n):
                X = tuple([[1,2,3,4],[1,2,3,4]])
                B = X[0:1]
                B[0][3]=n
                return B, X'''
        self.run_test(code, 20, tuple_slicing0=[int])

    def test_tuple_slicing1(self):
        code = '''
            def tuple_slicing1(n):
                X = tuple([[1,2,3,4],[1,2,3,4], [5,6,7,8]])
                B = X[0::2]
                B[0][3]=n
                return B, X'''
        self.run_test(code, 20, tuple_slicing1=[int])

    def test_reserved_identifier0(self):
        code = '''
        def reserved_identifier0(x):
            if x == 1:
                case = 1
            else:
                case = 2
            return case'''
        self.run_test(code, 3, reserved_identifier0=[int])

    def test_global_effects_partial0(self):
        code = '''
g = [1, 2]

def return_partial(x):
    def partial(_):
        return x

    return partial

def call_partial(fct):
    return return_partial(fct)

all_commands = call_partial(g)

def global_effects_partial0(l):
    return all_commands(l)'''
        self.run_test(code, 3, global_effects_partial0=[int])

    def test_dynamic_tuple_compare(self):
        code = '''
           def dynamic_tuple_compare(x, y):
               y = tuple(y)
               x = tuple(x)
               return x < y, x <= y, x > y, x >= y'''
        self.run_test(code, [1, 2], [1, 3], dynamic_tuple_compare=[List[int], List[int]])

    def test_annotations(self):
        code  = '''
        import numpy as np
        def annotations(x: np.ndarray) -> np.ndarray:
            return x'''
        self.run_test(code, numpy.ones(1), annotations=[NDArray[float, :]])

    def test_tuple_indexable_container(self):
        code = """
import numpy as np
def A_I():
    s = np.array([0.70817816, 0.68863678], dtype=np.float64)
    m = np.array([-1.11312199, -0.99629629], dtype=np.float64)
    self = tuple([100, s, m])
    return self

def A_F1(self, Input_x):
    Input_x = (Input_x - self[2]) / self[1]
    return Input_x

def B_I():
    self = tuple([0.0, 0])
    return self


def C_I(sRate):
    a = A_I()
    b = B_I()
    self = tuple([0, sRate, 0, a, b])
    return self


def C_Test(self):
    F = np.zeros((1, 1, 1))
    ret = A_F1(self[3], F.astype(np.float32))
    return ret

def tuple_indexable_container(n):
    TD = C_I(n)
    return C_Test(TD)"""
        self.run_test(code, 44100, tuple_indexable_container=[int])



#from http://jakevdp.github.com/blog/2012/08/24/numba-vs-cython/
#runas X = [ [i/100.+j for i in range(100) ] for j in range(30) ] ; pairwise(X)
#bench X = [ [i/100.+j for i in range(800) ] for j in range(100) ] ; pairwise(X)
#pythran export pairwise(float list list)

import math
def pairwise(X):
    M = len(X)
    N = len(X[0])
    D = [ [0 for x in range(M) ] for y in range(M) ]
    "omp parallel for"
    for i in range(M):
        for j in range(M):
            d = 0.0
            for k in range(N):
                tmp = X[i][k] - X[j][k]
                d += tmp * tmp
            D[i][j] = math.sqrt(d)
    return D



#from https://github.com/iskandr/parakeet/blob/master/benchmarks/nd_local_maxima.py
#pythran export local_maxima(float [][][][])
#runas import numpy as np ; shape = (8,6,4,2) ; x = np.arange(8*6*4*2, dtype=np.float64).reshape(*shape) ; local_maxima(x)

import numpy as np

def wrap(pos, offset, bound):
    return ( pos + offset ) % bound

def clamp(pos, offset, bound):
    return min(bound-1,max(0,pos+offset))

def reflect(pos, offset, bound):
    idx = pos+offset
    return min(2*(bound-1)-idx,max(idx,-idx))


def local_maxima(data, mode=wrap):
  wsize = data.shape
  result = np.ones(data.shape, bool)
  for pos in np.ndindex(data.shape):
    myval = data[pos]
    for offset in np.ndindex(wsize):
      neighbor_idx = tuple(mode(p, o-w//2, w) for (p, o, w) in zip(pos, offset, wsize))
      result[pos] &= (data[neighbor_idx] <= myval)
  return result



#unittest.python3.skip string.find not available anymore.
#pythran export score_text(str, str:int dict)
#runas score_text("e", { "d": 1 })
import string

def score_text(txt,kwdict):

	score=0
	for kw in kwdict.keys():
		if string.find(txt,kw)>-1:
			score+=kwdict[kw]
	return score



def omp_parallel_sections_lastprivate():
    sum =0
    sum0 = 0
    i0 = -1
    if 'omp parallel sections private(sum0) lastprivate(i0) private(i) num_threads(4)':
        if 'omp section':
            sum0 = 0
            for i in range(1, 400):
                sum0 += i
                i0 = i
            if 'omp critical(lock)':
                sum += sum0
        if 'omp section':
            sum0 = 0
            for i in range(400, 700):
                sum0 += i
                i0 = i
            if 'omp critical(lock)':
                sum += sum0
        if 'omp section':
            sum0 = 0
            for i in range(700, 1000):
                sum0 += i
                i0 = i
            if 'omp critical(lock)':
                sum += sum0

    known_sum = (999 * 1000) / 2
    return known_sum == sum and i0 == 999



#pythran export compute_mask(int[:,:], int[:,:])
#runas import numpy as np; coords = np.array([[0, 0, 1, 1, 2, 2]]); indices = np.array([[0, 3, 2]]); compute_mask(coords, indices)

import numpy as np
def compute_mask(coords, indices):  # pragma: no cover
    """
    Gets the mask for the coords given the indices in slice format.

    Works with either start-stop ranges of matching indices into coords
    called "pairs" (start-stop pairs) or filters the mask directly, based
    on which is faster.

    Exploits the structure in sorted coords, which is that for a constant
    value of coords[i - 1], coords[i - 2] and so on, coords[i] is sorted.
    Concretely, ``coords[i, coords[i - 1] == v1 & coords[i - 2] = v2, ...]``
    is always sorted. It uses this sortedness to find sub-pairs for each
    dimension given the previous, and so on. This is efficient for small
    slices or ints, but not for large ones.

    After it detects that working with pairs is rather inefficient (or after
    going through each possible index), it constructs a filtered mask from the
    start-stop pairs.

    Parameters
    ----------
    coords : np.ndarray
        The coordinates of the array.
    indices : np.ndarray
        The indices in the form of slices such that indices[:, 0] are starts,
        indices[:, 1] are stops and indices[:, 2] are steps.

    Returns
    -------
    mask : np.ndarray
        The starts and stops in the mask.
    is_slice : bool
        Whether or not the array represents a continuous slice.

    Examples
    --------
    Let's create some mock coords and indices

    >>> import numpy as np
    >>> coords = np.array([[0, 0, 1, 1, 2, 2]])
    >>> indices = np.array([[0, 3, 2]])  # Equivalent to slice(0, 3, 2)

    Now let's get the mask. Notice that the indices of ``0`` and ``2`` are matched.

    >>> _compute_mask(coords, indices)
    (array([0, 1, 4, 5]), False)

    Now, let's try with a more "continuous" slice. Matches ``0`` and ``1``.

    >>> indices = np.array([[0, 2, 1]])
    >>> _compute_mask(coords, indices)
    (array([0, 4]), True)

    This is equivalent to mask being ``slice(0, 4, 1)``.
    """
    # Set the initial mask to be the entire range of coordinates.
    starts = [0]
    stops = [coords.shape[1]]
    n_matches = coords.shape[1]

    i = 0
    while i < len(indices):
        # Guesstimate whether working with pairs is more efficient or
        # working with the mask directly.
        # One side is the estimate of time taken for binary searches
        # (n_searches * log(avg_length))
        # The other is an estimated time of a linear filter for the mask.
        n_pairs = len(starts)
        n_current_slices = _get_slice_len(indices[i]) * n_pairs + 2
        if n_current_slices * np.log(n_current_slices / max(n_pairs, 1)) > \
                n_matches + n_pairs:
            break

        # For each of the pairs, search inside the coordinates for other
        # matching sub-pairs.
        # This gets the start-end coordinates in coords for each 'sub-array'
        # Which would come out of indexing a single integer.
        starts, stops, n_matches = _get_mask_pairs(starts, stops, coords[i], indices[i])

        i += 1

    # Combine adjacent pairs
    starts, stops = _join_adjacent_pairs(starts, stops)

    # If just one pair is left over, treat it as a slice.
    if i == len(indices) and len(starts) == 1:
        return np.array([starts[0], stops[0]]), True

    # Convert start-stop pairs into mask, filtering by remaining
    # coordinates.
    mask = _filter_pairs(starts, stops, coords[i:], indices[i:])

    return np.array(mask, dtype=np.intp), False


def _get_slice_len(idx):
    """
    Get the number of elements in a slice.

    Parameters
    ----------
    idx : np.ndarray
        A (3,) shaped array containing start, stop, step

    Returns
    -------
    n : int
        The length of the slice.

    Examples
    --------
    >>> idx = np.array([5, 15, 5])
    >>> _get_slice_len(idx)
    2
    """
    start, stop, step = idx[0], idx[1], idx[2]

    if step > 0:
        return (stop - start + step - 1) // step
    else:
        return (start - stop - step - 1) // (-step)


def _get_mask_pairs(starts_old, stops_old, c, idx):  # pragma: no cover
    """
    Gets the pairs for a following dimension given the pairs for
    a dimension.

    For each pair, it searches in the following dimension for
    matching coords and returns those.

    The total combined length of all pairs is returned to
    help with the performance guesstimate.

    Parameters
    ----------
    starts_old, stops_old : list[int]
        The starts and stops from the previous index.
    c : np.ndarray
        The coords for this index's dimension.
    idx : np.ndarray
        The index in the form of a slice.
        idx[0], idx[1], idx[2] = start, stop, step

    Returns
    -------
    starts, stops: list
        The starts and stops after applying the current index.
    n_matches : int
        The sum of elements in all ranges.

    Examples
    --------
    >>> c = np.array([1, 2, 1, 2, 1, 1, 2, 2])
    >>> starts_old = [4]
    >>> stops_old = [8]
    >>> idx = np.array([1, 2, 1])
    >>> _get_mask_pairs(starts_old, stops_old, c, idx)
    ([4], [6], 2)
    """
    starts = []
    stops = []
    n_matches = 0

    for j in range(len(starts_old)):
        # For each matching "integer" in the slice, search within the "sub-coords"
        # Using binary search.
        for p_match in range(idx[0], idx[1], idx[2]):
            start = np.searchsorted(c[starts_old[j]:stops_old[j]], p_match) + starts_old[j]
            stop = np.searchsorted(c[starts_old[j]:stops_old[j]], p_match + 1) + starts_old[j]

            if start != stop:
                starts.append(start)
                stops.append(stop)
                n_matches += stop - start

    return starts, stops, n_matches

def _join_adjacent_pairs(starts_old, stops_old):  # pragma: no cover
    """
    Joins adjacent pairs into one. For example, 2-5 and 5-7
    will reduce to 2-7 (a single pair). This may help in
    returning a slice in the end which could be faster.

    Parameters
    ----------
    starts_old, stops_old : list[int]
        The input starts and stops

    Returns
    -------
    starts, stops : list[int]
        The reduced starts and stops.

    Examples
    --------
    >>> starts = [2, 5]
    >>> stops = [5, 7]
    >>> _join_adjacent_pairs(starts, stops)
    ([2], [7])
    """
    if len(starts_old) <= 1:
        return starts_old, stops_old

    starts = [starts_old[0]]
    stops = []

    for i in range(1, len(starts_old)):
        if starts_old[i] != stops_old[i - 1]:
            starts.append(starts_old[i])
            stops.append(stops_old[i - 1])

    stops.append(stops_old[-1])

    return starts, stops

def _filter_pairs(starts, stops, coords, indices):  # pragma: no cover
    """
    Converts all the pairs into a single integer mask, additionally filtering
    by the indices.

    Parameters
    ----------
    starts, stops : list[int]
        The starts and stops to convert into an array.
    coords : np.ndarray
        The coordinates to filter by.
    indices : np.ndarray
        The indices in the form of slices such that indices[:, 0] are starts,
        indices[:, 1] are stops and indices[:, 2] are steps.

    Returns
    -------
    mask : list
        The output integer mask.

    Examples
    --------
    >>> import numpy as np
    >>> starts = [2]
    >>> stops = [7]
    >>> coords = np.array([[0, 1, 2, 3, 4, 5, 6, 7]])
    >>> indices = np.array([[2, 8, 2]]) # Start, stop, step pairs
    >>> _filter_pairs(starts, stops, coords, indices)
    [2, 4, 6]
    """
    mask = []

    # For each pair,
    for i in range(len(starts)):
        # For each element match within the pair range
        for j in range(starts[i], stops[i]):
            match = True

            # Check if it matches all indices
            for k in range(len(indices)):
                idx = indices[k]
                elem = coords[k, j]

                match &= ((elem - idx[0]) % idx[2] == 0 and
                          ((idx[2] > 0 and idx[0] <= elem < idx[1])
                           or (idx[2] < 0 and idx[0] >= elem > idx[1])))

            # and append to the mask if so.
            if match:
                mask.append(j)

    return mask



#runas solve()
#unittest.skip type can't be deducte
#pythran export solve()
def solve():
    '''
    In the card game poker, a hand consists of five cards and are ranked, from lowest to highest, in the following way:
    
    High Card: Highest value card.
    One Pair: Two cards of the same value.
    Two Pairs: Two different pairs.
    Three of a Kind: Three cards of the same value.
    Straight: All cards are consecutive values.
    Flush: All cards of the same suit.
    Full House: Three of a kind and a pair.
    Four of a Kind: Four cards of the same value.
    Straight Flush: All cards are consecutive values of same suit.
    Royal Flush: Ten, Jack, Queen, King, Ace, in same suit.
    The cards are valued in the order:
    2, 3, 4, 5, 6, 7, 8, 9, 10, Jack, Queen, King, Ace.
    
    If two players have the same ranked hands then the rank made up of the highest value wins; for example, a pair of eights beats a pair of fives (see example 1 below). But if two ranks tie, for example, both players have a pair of queens, then highest cards in each hand are compared (see example 4 below); if the highest cards tie then the next highest cards are compared, and so on.
    
    Consider the following five hands dealt to two players:
    
    Hand        Player 1            Player 2          Winner
    1       5H 5C 6S 7S KD      2C 3S 8S 8D TD      Player 2
            Pair of Fives       Pair of Eights
    
    2       5D 8C 9S JS AC      2C 5C 7D 8S QH      Player 1
            Highest card Ace    Highest card Queen
    
    3       2D 9C AS AH AC      3D 6D 7D TD QD      Player 2
            Three Aces          Flush with Diamonds
    
    4       4D 6S 9H QH QC      3D 6D 7H QD QS      Player 1
            Pair of Queens      Pair of Queens
            Highest card Nine   Highest card Seven
    
    5       2H 2D 4C 4D 4S      3C 3D 3S 9S 9D      Player 1
            Full House          Full House
            With Three Fours    with Three Threes
    
    The file, poker.txt, contains one-thousand random hands dealt to two players. Each line of the file contains ten cards (separated by a single space): the first five are Player 1's cards and the last five are Player 2's cards. You can assume that all hands are valid (no invalid characters or repeated cards), each player's hand is in no specific order, and in each hand there is a clear winner.
    
    How many hands does Player 1 win?
    '''
    
    value = { '2':2,'3':3,'4':4,'5':5,'6':6,'7':7,'8':8,'9':9,'T':10,'J':11,'Q':12,'K':13,'A':14 }
    all_kinds = tuple(reversed(sorted(value.values())))
    all_suits = list('DCSH')
    
    def make_hand(cards):
        hand = {}
        for card in cards:
            hand.setdefault(value[card[0]], {})[card[1]] = 1
            hand.setdefault(card[1], {})[value[card[0]]] = 1
        return hand
    
    def get(hash, arr): return ((i, hash.get(i, {})) for i in arr)
    def has(hash, arr): return not sum(1 for i in arr if i not in hash)
    
    def rank(hand):
        # Royal flush
        for suit, kinds in get(hand, all_suits):
            if has(kinds, tuple('TJQKA')):
                return (9,0,0)
    
        # Straight flush
        for suit, kinds in get(hand, all_suits):
            kinds = sorted(kind for kind in kinds.keys())
            if len(kinds) == 5 and kinds[4] - kinds[0] == 4:
                return (8, kinds[0],0)
    
        # Four of a kind
        for kind, suits in get(hand, all_kinds):
            if len(suits.keys()) == 4:
                return (7, kind,0)
    
        # Full house
        for kind, suits in get(hand, all_kinds):
            if len(suits.keys()) == 3:
                for kind2, suits2 in get(hand, all_kinds):
                    if len(suits2.keys()) == 2:
                        return (6, kind, kind2)
    
        # Flush
        for suit, kinds in get(hand, all_suits):
            if len(kinds.keys()) == 5:
                return (5,0,0)
    
        # Straight
        kinds = sorted(kind for kind in all_kinds if kind in hand)
        if len(kinds) == 5 and kinds[4] - kinds[0] == 4:
            return (4, kinds[0],0)
    
        # Three of a kind
        for kind, suits in get(hand, all_kinds):
            if len(suits.keys()) == 3:
                return (3, kind,0)
    
        # Two pairs
        for kind, suits in get(hand, all_kinds):
            if len(suits.keys()) == 2:
                for kind2, suits2 in get(hand, all_kinds):
                    if kind != kind2 and len(suits2.keys()) == 2:
                        return (2, kind, kind2)
    
        # One pair
        for kind, suits in get(hand, all_kinds):
            if len(suits.keys()) == 2:
                return (1, kind,0)
    
        for kind in all_kinds:
            if kind in hand:
                return (0, kind,0)
    
        return (0,0,0)
    
    
    count = 0
    for hand in open('poker.txt'):
        hands = hand.split(' ')
        p1, p2 = make_hand(hands[0:5]), make_hand(hands[5:10])
        v1, v2 = rank(p1), rank(p2)
        if v1 > v2: count += 1
    return count




#skip.pythran export dilate_decompose(int[][], int)
#pythran export dilate_decompose_loops(float[][], int)
#skip.pythran export dilate_decompose_interior(int[][], int[][])
#skip.runas import numpy as np ; image = np.random.randint(0, 256,  (width, height)) / 256.0 ; dilate_decompose_loops(image)
#runas import numpy as np ; image = np.tri(100, 200) /2.0 ; dilate_decompose_loops(image, 4)
#bench import numpy as np ; image = np.tri(500, 600) /2.0 ; dilate_decompose_loops(image, 4)

from numpy import empty_like

def dilate_decompose_loops(x, k):
  m,n = x.shape
  y = empty_like(x)
  for i in range(m):
    for j in range(n):
      left_idx = max(0, i-k//2)
      right_idx = min(m, i+k//2+1)
      currmax = x[left_idx, j]
      for ii in range(left_idx+1, right_idx):
        elt = x[ii, j]
        if elt > currmax:
          currmax = elt
      y[i, j] = currmax
  z = empty_like(x)
  for i in range(m):
    for j in range(n):
      left_idx = max(0, j-k//2)
      right_idx = min(n, j+k//2+1)
      currmax = y[i,left_idx]
      for jj in range(left_idx+1, right_idx):
        elt = y[i,jj]
        if elt > currmax:
          currmax = elt
      z[i,j] = currmax
  return z

#def dilate_1d_naive(x_strip,  k):
#  """
#  Given a 1-dimensional input and 1-dimensional output,
#  fill output with 1d dilation of input
#  """
#  nelts = len(x_strip)
#  y_strip = empty_like(x_strip)
#  half = k / 2
#  for idx in range(nelts):
#    left_idx = max(idx-half,0)
#    right_idx = min(idx+half+1, nelts)
#    currmax = x_strip[left_idx]
#    for j in range(left_idx+1, right_idx):
#      elt = x_strip[j]
#      if elt > currmax:
#        currmax = elt
#    y_strip[idx] = currmax
#  return y_strip
#
#def dilate_decompose(x, k):
#  import numpy as np
#  m,n = x.shape
#  y = np.array([dilate_1d_naive(x[row_idx, :], k) for row_idx in range(m)])
#  return np.array([dilate_1d_naive(y[:, col_idx], k) for col_idx in range(n)]).T
#
#def dilate_1d_interior(x_strip, k):
#
#  nelts = len(x_strip)
#  y_strip = empty_like(x_strip)
#  half = k / 2
#
#  interior_start = half+1
#  interior_stop = max(nelts-half, interior_start)
#
#  # left boundary
#  for i in range(min(half+1, nelts)):
#    left_idx = max(i-half,0)
#    right_idx = min(i+half+1, nelts)
#    currmax = x_strip[left_idx]
#    for j in range(left_idx+1, right_idx):
#      elt = x_strip[j]
#      if elt > currmax:
#        currmax = elt
#    y_strip[i] = currmax
#
#  #interior
#  for i in range(interior_start, interior_stop):
#    left_idx = i-half
#    right_idx = i+half+1
#    currmax = x_strip[left_idx]
#    for j in range(left_idx+1, right_idx):
#      elt = x_strip[j]
#      if elt > currmax:
#        currmax = elt
#    y_strip[i] = currmax
#
#  # right boundary
#  for i in range(interior_stop, nelts):
#    left_idx = max(i-half, 0)
#    right_idx = nelts
#    currmax = x_strip[left_idx]
#    for j in range(left_idx+1, right_idx):
#      elt = x_strip[j]
#      if elt > currmax:
#        currmax = elt
#    y_strip[i] = currmax
#  return y_strip
#
#def dilate_decompose_interior(x, k):
#  m,n = x.shape
#  y = np.array([dilate_1d_interior(x[row_idx, :],k) for row_idx in range(m)])
#  return np.array([dilate_1d_interior(y[:, col_idx],k) for col_idx in range(n)]).T



#from http://continuum.io/blog/numba_performance
#runas test(50)
#bench test(28)

import math
import numpy as np
def window_floor(idx, radius):
    if radius > idx:
        return 0
    else:
        return idx - radius


def window_ceil(idx, ceil, radius):
    if idx + radius > ceil:
        return ceil
    else:
        return idx + radius

def python_kernel(image, state, state_next, window_radius):
    changes = 0
    sqrt_3 = math.sqrt(3.0)

    height = image.shape[0]
    width = image.shape[1]

    for j in range(width):
        for i in range(height):

            winning_colony = state[i, j, 0]
            defense_strength = state[i, j, 1]

            for jj in range(window_floor(j, window_radius),
                             window_ceil(j+1, width, window_radius)):
                for ii in range(window_floor(i, window_radius),
                                 window_ceil(i+1, height, window_radius)):
                    if (ii == i and jj == j):
                        continue

                    d = image[i, j, 0] - image[ii, jj, 0]
                    s = d * d
                    for k in range(1, 3):
                        d = image[i, j, k] - image[ii, jj, k]
                        s += d * d
                    gval = 1.0 - math.sqrt(s)/sqrt_3

                    attack_strength = gval * state[ii, jj, 1]

                    if attack_strength > defense_strength:
                        defense_strength = attack_strength
                        winning_colony = state[ii, jj, 0]
                        changes += 1

            state_next[i, j, 0] = winning_colony
            state_next[i, j, 1] = defense_strength

    return changes

#pythran export test(int)
def test(N):
    image = np.zeros((N, N, 3))
    state = np.zeros((N, N, 2))
    state_next = np.empty_like(state)


    # colony 1 is strength 1 at position 0,0
    # colony 0 is strength 0 at all other positions
    state[0, 0, 0] = 1
    state[0, 0, 1] = 1
    return python_kernel(image, state, state_next, 10)
